@@ -1,14 +1,17 @@
 import abc
+import importlib.metadata
 import inspect
 
-import entrypoints
-from mlflow.exceptions import MlflowException
-from mlflow.protos.databricks_pb2 import RESOURCE_DOES_NOT_EXIST, INTERNAL_ERROR
+import importlib_metadata
+
 from mlflow.deployments.base import BaseDeploymentClient
 from mlflow.deployments.utils import parse_target_uri
+from mlflow.exceptions import MlflowException
+from mlflow.protos.databricks_pb2 import INTERNAL_ERROR, RESOURCE_DOES_NOT_EXIST
 from mlflow.utils.annotations import developer_stable
+from mlflow.utils.plugins import get_entry_points
 
-# TODO: refactor to have a common base class for all the plugin implementation in MLFlow
+# TODO: refactor to have a common base class for all the plugin implementation in MLflow
 #   mlflow/tracking/context/registry.py
 #   mlflow/tracking/registry
 #   mlflow/store/artifact/artifact_repository_registry.py
@@ -24,7 +27,6 @@ class PluginManager(abc.ABC):
     registered through the `register_entrypoints` method.
     """
 
-    @abc.abstractmethod
     def __init__(self, group_name):
         self._registry = {}
         self.group_name = group_name
@@ -54,22 +56,23 @@ class PluginManager(abc.ABC):
         return self._has_registered
 
     def register(self, target_name, plugin_module):
+        """Register a deployment client given its target name and module
+        Args:
+            target_name: The name of the deployment target. This name will be used by
+                `get_deploy_client()` to retrieve a deployment client from
+                the plugin store.
+            plugin_module: The module that implements the deployment plugin interface.
         """
-        Register a deployment client given its target name and module
-
-        :param target_name: The name of the deployment target. This name will be used by
-                            `get_deploy_client()` to retrieve a deployment client from
-                            the plugin store
-        :param plugin_module: The module that implements the deployment plugin interface
-        """
-        self.registry[target_name] = entrypoints.EntryPoint(target_name, plugin_module, None)
+        self.registry[target_name] = importlib.metadata.EntryPoint(
+            target_name, plugin_module, self.group_name
+        )
 
     def register_entrypoints(self):
         """
         Runs through all the packages that has the `group_name` defined as the entrypoint
         and register that into the registry
         """
-        for entrypoint in entrypoints.get_group_all(self.group_name):
+        for entrypoint in get_entry_points(self.group_name):
             self.registry[entrypoint.name] = entrypoint
         self._has_registered = True
 
@@ -95,11 +98,11 @@ class DeploymentPlugins(PluginManager):
             )
             raise MlflowException(msg, error_code=RESOURCE_DOES_NOT_EXIST)
 
-        if isinstance(plugin_like, entrypoints.EntryPoint):
+        if isinstance(plugin_like, (importlib_metadata.EntryPoint, importlib.metadata.EntryPoint)):
             try:
                 plugin_obj = plugin_like.load()
             except (AttributeError, ImportError) as exc:
-                raise RuntimeError('Failed to load the plugin "{}": {}'.format(item, str(exc)))
+                raise RuntimeError(f'Failed to load the plugin "{item}": {exc}')
             self.registry[item] = plugin_obj
         else:
             plugin_obj = plugin_like
