@@ -1,12 +1,13 @@
 import logging
+
 import click
 
-from mlflow.models import build_docker as build_docker_api
+from mlflow.models import python_api
 from mlflow.models.flavor_backend_registry import get_flavor_backend
+from mlflow.models.model import update_model_requirements
 from mlflow.utils import cli_args
 from mlflow.utils import env_manager as _EnvManager
 
-
 _logger = logging.getLogger(__name__)
 
 
@@ -18,7 +19,6 @@ def commands():
     To deploy a model associated with a run on a tracking server, set the MLFLOW_TRACKING_URI
     environment variable to the URL of the desired server.
     """
-    pass
 
 
 @commands.command("serve")
@@ -97,10 +97,7 @@ def serve(
         }'
 
     """
-    if no_conda:
-        env_manager = _EnvManager.LOCAL
-    else:
-        env_manager = env_manager or _EnvManager.VIRTUALENV
+    env_manager = _EnvManager.LOCAL if no_conda else env_manager
 
     return get_flavor_backend(
         model_uri, env_manager=env_manager, workers=workers, install_mlflow=install_mlflow
@@ -109,6 +106,15 @@ def serve(
     )
 
 
+class KeyValueType(click.ParamType):
+    name = "key=value"
+
+    def convert(self, value, param, ctx):
+        if "=" not in value:
+            self.fail(f"{value!r} is not a valid key value pair, expecting `key=value`", param, ctx)
+        return value.split("=", 1)
+
+
 @commands.command("predict")
 @cli_args.MODEL_URI
 @click.option(
@@ -128,27 +134,47 @@ def serve(
 )
 @cli_args.ENV_MANAGER
 @cli_args.INSTALL_MLFLOW
+@click.option(
+    "--pip-requirements-override",
+    "-r",
+    default=None,
+    help="Specify packages and versions to override the dependencies defined "
+    "in the model. Must be a comma-separated string like x==y,z==a.",
+)
+@click.option(
+    "--env",
+    default=None,
+    type=KeyValueType(),
+    multiple=True,
+    help="Extra environment variables to set when running the model. Must be "
+    "key value pairs, e.g. `--env key=value`.",
+)
 def predict(
     model_uri,
-    input_path,
-    output_path,
-    content_type,
-    env_manager,
-    install_mlflow,
+    input_data=None,
+    input_path=None,
+    content_type=python_api._CONTENT_TYPE_JSON,
+    output_path=None,
+    env_manager=_EnvManager.VIRTUALENV,
+    install_mlflow=False,
+    pip_requirements_override=None,
+    env=None,
 ):
     """
     Generate predictions in json format using a saved MLflow model. For information about the input
     data formats accepted by this function, see the following documentation:
     https://www.mlflow.org/docs/latest/models.html#built-in-deployment-tools.
     """
-    env_manager = env_manager or _EnvManager.VIRTUALENV
-    return get_flavor_backend(
-        model_uri, env_manager=env_manager, install_mlflow=install_mlflow
-    ).predict(
+    return python_api.predict(
         model_uri=model_uri,
+        input_data=input_data,
         input_path=input_path,
-        output_path=output_path,
         content_type=content_type,
+        output_path=output_path,
+        env_manager=env_manager,
+        install_mlflow=install_mlflow,
+        pip_requirements_override=pip_requirements_override,
+        extra_envs=dict(env),
     )
 
 
@@ -166,7 +192,6 @@ def prepare_env(
     downloading dependencies or initializing a conda environment. After preparation,
     calling predict or serve should be fast.
     """
-    env_manager = env_manager or _EnvManager.VIRTUALENV
     return get_flavor_backend(
         model_uri, env_manager=env_manager, install_mlflow=install_mlflow
     ).prepare_env(model_uri=model_uri)
@@ -180,12 +205,19 @@ def prepare_env(
     default="mlflow-dockerfile",
     help="Output directory where the generated Dockerfile is stored.",
 )
-@cli_args.ENV_MANAGER
+@cli_args.ENV_MANAGER_DOCKERFILE
 @cli_args.MLFLOW_HOME
+@cli_args.INSTALL_JAVA
 @cli_args.INSTALL_MLFLOW
 @cli_args.ENABLE_MLSERVER
 def generate_dockerfile(
-    model_uri, output_directory, env_manager, mlflow_home, install_mlflow, enable_mlserver
+    model_uri,
+    output_directory,
+    env_manager,
+    mlflow_home,
+    install_java,
+    install_mlflow,
+    enable_mlserver,
 ):
     """
     Generates a directory with Dockerfile whose default entrypoint serves an MLflow model at port
@@ -197,13 +229,13 @@ def generate_dockerfile(
         _logger.info("Generating Dockerfile for model %s", model_uri)
     else:
         _logger.info("Generating Dockerfile")
-    env_manager = env_manager or _EnvManager.CONDA
     backend = get_flavor_backend(model_uri, docker_build=True, env_manager=env_manager)
     if backend.can_build_image():
         backend.generate_dockerfile(
             model_uri,
             output_directory,
             mlflow_home=mlflow_home,
+            install_java=install_java,
             install_mlflow=install_mlflow,
             enable_mlserver=enable_mlserver,
         )
@@ -221,9 +253,10 @@ def generate_dockerfile(
 @click.option("--name", "-n", default="mlflow-pyfunc-servable", help="Name to use for built image")
 @cli_args.ENV_MANAGER
 @cli_args.MLFLOW_HOME
+@cli_args.INSTALL_JAVA
 @cli_args.INSTALL_MLFLOW
 @cli_args.ENABLE_MLSERVER
-def build_docker(model_uri, name, env_manager, mlflow_home, install_mlflow, enable_mlserver):
+def build_docker(**kwargs):
     """
     Builds a Docker image whose default entrypoint serves an MLflow model at port 8080, using the
     python_function flavor. The container serves the model referenced by ``--model-uri``, if
@@ -250,6 +283,13 @@ def build_docker(model_uri, name, env_manager, mlflow_home, install_mlflow, enab
         # Mount the model stored in '/local/path/to/artifacts/model' and serve it
         docker run --rm -p 5001:8080 -v /local/path/to/artifacts/model:/opt/ml/model "my-image-name"
 
+    .. important::
+
+        Since MLflow 2.10.1, the Docker image built with ``--model-uri`` does **not install Java**
+        for improved performance, unless the model flavor is one of ``["johnsnowlabs", "h2o",
+        "mleap", "spark"]``. If you need to install Java for other flavors, e.g. custom Python model
+        that uses SparkML, please specify the ``--install-java`` flag to enforce Java installation.
+
     .. warning::
 
         The image built without ``--model-uri`` doesn't support serving models with RFunc / Java
@@ -266,12 +306,47 @@ def build_docker(model_uri, name, env_manager, mlflow_home, install_mlflow, enab
     See https://www.mlflow.org/docs/latest/python_api/mlflow.pyfunc.html for more information on the
     'python_function' flavor.
     """
-    env_manager = env_manager or _EnvManager.VIRTUALENV
-    build_docker_api(
-        model_uri,
-        name,
-        env_manager=env_manager,
-        mlflow_home=mlflow_home,
-        install_mlflow=install_mlflow,
-        enable_mlserver=enable_mlserver,
-    )
+    python_api.build_docker(**kwargs)
+
+
+@commands.command("update-pip-requirements")
+@cli_args.MODEL_URI
+@click.argument("operation", type=click.Choice(["add", "remove"]))
+@click.argument("requirement_strings", type=str, nargs=-1)
+def update_pip_requirements(model_uri, operation, requirement_strings):
+    """
+    Add or remove requirements from a model's conda.yaml and requirements.txt files.
+    If using a remote tracking server, please make sure to set the MLFLOW_TRACKING_URI
+    environment variable to the URL of the desired server.
+
+    REQUIREMENT_STRINGS is a list of pip requirements specifiers.
+    See below for examples.
+
+    Sample usage:
+
+    .. code::
+
+        # Add requirements using the model's "runs:/" URI
+
+        mlflow models update-pip-requirements -m runs:/<run_id>/<model_path> \\
+            add "pandas==1.0.0" "scikit-learn" "mlflow >= 2.8, != 2.9.0"
+
+        # Remove requirements from a local model
+
+        mlflow models update-pip-requirements -m /path/to/local/model \\
+            remove "torchvision" "pydantic"
+
+    Note that model registry URIs (i.e. URIs in the form ``models:/``) are not
+    supported, as artifacts in the model registry are intended to be read-only.
+    Editing requirements is read-only artifact repositories is also not supported.
+
+    If adding requirements, the function will overwrite any existing requirements
+    that overlap, or else append the new requirements to the existing list.
+
+    If removing requirements, the function will ignore any version specifiers,
+    and remove all the specified package names. Any requirements that are not
+    found in the existing files will be ignored.
+    """
+    update_model_requirements(model_uri, operation, requirement_strings)
+
+    _logger.info(f"Successfully updated the requirements for the model at {model_uri}!")
