@@ -1,20 +1,21 @@
-from packaging.version import Version
 import logging
-import mlflow.pytorch
 import os
-import shutil
 import tempfile
 import warnings
 
+from packaging.version import Version
+
+import mlflow.pytorch
 from mlflow.exceptions import MlflowException
 from mlflow.ml_package_versions import _ML_PACKAGE_VERSIONS
-from mlflow.pytorch import _pytorch_autolog
 from mlflow.utils.autologging_utils import (
-    ExceptionSafeAbstractClass,
     BatchMetricsLogger,
+    ExceptionSafeAbstractClass,
     MlflowAutologgingQueueingClient,
+    disable_autologging,
     get_autologging_config,
 )
+from mlflow.utils.checkpoint_utils import MlflowModelCheckpointCallbackBase
 
 logging.basicConfig(level=logging.ERROR)
 MIN_REQ_VERSION = Version(_ML_PACKAGE_VERSIONS["pytorch-lightning"]["autologging"]["minimum"])
@@ -23,7 +24,6 @@ MAX_REQ_VERSION = Version(_ML_PACKAGE_VERSIONS["pytorch-lightning"]["autologging
 import pytorch_lightning as pl
 from pytorch_lightning.utilities import rank_zero_only
 
-
 # The following are the downsides of using PyTorch Lightning's built-in MlflowLogger.
 # 1. MlflowLogger doesn't provide a mechanism to store an entire model into mlflow.
 #    Only model checkpoint is saved.
@@ -35,19 +35,18 @@ from pytorch_lightning.utilities import rank_zero_only
 # TODO: Replace __MlflowPLCallback with Pytorch Lightning's built-in MlflowLogger
 # once the above mentioned issues have been addressed
 
+_logger = logging.getLogger(__name__)
 
 _pl_version = Version(pl.__version__)
 if _pl_version < Version("1.5.0"):
-    # pylint: disable-next=ungrouped-imports
     from pytorch_lightning.core.memory import ModelSummary
 else:
-    # pylint: disable-next=ungrouped-imports
     from pytorch_lightning.utilities.model_summary import ModelSummary
 
 
 def _get_optimizer_name(optimizer):
     """
-    In pytorch-lightining 1.1.0, `LightningOptimizer` was introduced:
+    In pytorch-lightning 1.1.0, `LightningOptimizer` was introduced:
     https://github.com/PyTorchLightning/pytorch-lightning/pull/4658
 
     If a user sets `enable_pl_optimizer` to True when instantiating a `Trainer` object,
@@ -58,7 +57,6 @@ def _get_optimizer_name(optimizer):
     if Version(pl.__version__) < Version("1.1.0"):
         return optimizer.__class__.__name__
     else:
-        # pylint: disable-next=ungrouped-imports
         from pytorch_lightning.core.optimizer import LightningOptimizer
 
         return (
@@ -68,7 +66,7 @@ def _get_optimizer_name(optimizer):
         )
 
 
-class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
+class __MlflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
     """
     Callback for auto-logging metrics and parameters.
     """
@@ -87,6 +85,7 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         self.log_models = log_models
         self.log_every_n_epoch = log_every_n_epoch
         self.log_every_n_step = log_every_n_step
+        self._global_steps_per_training_step = 1
         # Sets for tracking which metrics are logged on steps and which are logged on epochs
         self._step_metrics = set()
         self._epoch_metrics = set()
@@ -134,9 +133,7 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
     if _pl_version >= Version("1.4.0dev"):
 
         @rank_zero_only
-        def on_train_epoch_end(
-            self, trainer, pl_module, *args
-        ):  # pylint: disable=signature-differs,arguments-differ,unused-argument
+        def on_train_epoch_end(self, trainer, pl_module, *args):
             self._log_epoch_metrics(trainer, pl_module)
 
     # In pytorch-lightning >= 1.2.0, logging metrics in `on_epoch_end` results in duplicate
@@ -151,14 +148,14 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         # pytorch-lightning >= 1.3.0
 
         @rank_zero_only
-        def on_train_epoch_end(
-            self, trainer, pl_module, *args
-        ):  # pylint: disable=signature-differs,arguments-differ,unused-argument
+        def on_train_epoch_end(self, trainer, pl_module, *args):
             """
             Log loss and other metrics values after each train epoch
 
-            :param trainer: pytorch lightning trainer instance
-            :param pl_module: pytorch lightning base module
+            Args:
+                trainer: pytorch lightning trainer instance
+                pl_module: pytorch lightning base module
+                args: additional positional arguments
             """
             # If validation loop is enabled (meaning `validation_step` is overridden),
             # log metrics in `on_validaion_epoch_end` to avoid logging the same metrics
@@ -171,8 +168,9 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
             """
             Log loss and other metrics values after each validation epoch
 
-            :param trainer: pytorch lightning trainer instance
-            :param pl_module: pytorch lightning base module
+            Args:
+                trainer: pytorch lightning trainer instance
+                pl_module: pytorch lightning base module
             """
             self._log_epoch_metrics(trainer, pl_module)
 
@@ -183,20 +181,21 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
             """
             Log loss and other metrics values after each epoch
 
-            :param trainer: pytorch lightning trainer instance
-            :param pl_module: pytorch lightning base module
+            Args:
+                trainer: pytorch lightning trainer instance
+                pl_module: pytorch lightning base module
             """
             self._log_epoch_metrics(trainer, pl_module)
 
     @rank_zero_only
-    def on_train_batch_end(
-        self, trainer, pl_module, *args
-    ):  # pylint: disable=signature-differs,arguments-differ,unused-argument
+    def on_train_batch_end(self, trainer, pl_module, *args):
         """
         Log metric values after each step
 
-        :param trainer: pytorch lightning trainer instance
-        :param pl_module: pytorch lightning base module
+        Args:
+            trainer: pytorch lightning trainer instance
+            pl_module: pytorch lightning base module
+            args: additional positional arguments
         """
         if not self.log_every_n_step:
             return
@@ -213,7 +212,7 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         ]
         self._step_metrics.update(name for (name, _) in metric_items)
         step = trainer.global_step
-        if (step + 1) % self.log_every_n_step == 0:
+        if ((step // self._global_steps_per_training_step) + 1) % self.log_every_n_step == 0:
             self._log_metrics(trainer, step, metric_items)
 
     @rank_zero_only
@@ -221,8 +220,9 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         """
         Logs Optimizer related metrics when the train begins
 
-        :param trainer: pytorch lightning trainer instance
-        :param pl_module: pytorch lightning base module
+        Args:
+            trainer: pytorch lightning trainer instance
+            pl_module: pytorch lightning base module
         """
         self.client.set_tags(self.run_id, {"Mode": "training"})
 
@@ -237,6 +237,10 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         #    to rename the optimizer parameters based on keys in default dictionary.
 
         if hasattr(trainer, "optimizers"):
+            # Lightning >= 1.6.0 increments the global step every time an optimizer is stepped.
+            # We assume every optimizer will be stepped in each training step.
+            if _pl_version >= Version("1.6.0"):
+                self._global_steps_per_training_step = len(trainer.optimizers)
             optimizer = trainer.optimizers[0]
             params["optimizer_name"] = _get_optimizer_name(optimizer)
 
@@ -251,8 +255,10 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         """
         Logs the model checkpoint into mlflow - models folder on the training end
 
-        :param trainer: pytorch lightning trainer instance
-        :param pl_module: pytorch lightning base module
+
+        Args:
+            trainer: pytorch lightning trainer instance
+            pl_module: pytorch lightning base module
         """
         # manually flush any remaining metadata from training
         self.metrics_logger.flush()
@@ -263,8 +269,9 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         """
         Logs accuracy and other relevant metrics on the testing end
 
-        :param trainer: pytorch lightning trainer instance
-        :param pl_module: pytorch lightning base module
+        Args:
+            trainer: pytorch lightning trainer instance
+            pl_module: pytorch lightning base module
         """
         self.client.set_tags(self.run_id, {"Mode": "testing"})
         self.client.flush(synchronous=True)
@@ -275,6 +282,110 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         self.metrics_logger.flush()
 
 
+class MlflowModelCheckpointCallback(pl.Callback, MlflowModelCheckpointCallbackBase):
+    """Callback for auto-logging pytorch-lightning model checkpoints to MLflow.
+    This callback implementation only supports pytorch-lightning >= 1.6.0.
+
+    Args:
+        monitor: In automatic model checkpointing, the metric name to monitor if
+            you set `model_checkpoint_save_best_only` to True.
+        save_best_only: If True, automatic model checkpointing only saves when
+            the model is considered the "best" model according to the quantity
+            monitored and previous checkpoint model is overwritten.
+        mode: one of {"min", "max"}. In automatic model checkpointing,
+            if save_best_only=True, the decision to overwrite the current save file is made
+            based on either the maximization or the minimization of the monitored quantity.
+        save_weights_only: In automatic model checkpointing, if True, then
+            only the model’s weights will be saved. Otherwise, the optimizer states,
+            lr-scheduler states, etc are added in the checkpoint too.
+        save_freq: `"epoch"` or integer. When using `"epoch"`, the callback
+            saves the model after each epoch. When using integer, the callback
+            saves the model at end of this many batches. Note that if the saving isn't
+            aligned to epochs, the monitored metric may potentially be less reliable (it
+            could reflect as little as 1 batch, since the metrics get reset
+            every epoch). Defaults to `"epoch"`.
+
+    .. code-block:: python
+        :caption: Example
+
+        import mlflow
+        from mlflow.pytorch import MLflowModelCheckpointCallback
+        from pytorch_lightning import Trainer
+
+        mlflow.pytorch.autolog(checkpoint=True)
+
+        model = MyLightningModuleNet()  # A custom-pytorch lightning model
+        train_loader = create_train_dataset_loader()
+
+        mlflow_checkpoint_callback = MLflowModelCheckpointCallback()
+
+        trainer = Trainer(callbacks=[mlflow_checkpoint_callback])
+
+        with mlflow.start_run() as run:
+            trainer.fit(model, train_loader)
+
+    """
+
+    def __init__(
+        self,
+        monitor="val_loss",
+        mode="min",
+        save_best_only=True,
+        save_weights_only=False,
+        save_freq="epoch",
+    ):
+        super().__init__(
+            checkpoint_file_suffix=".pth",
+            monitor=monitor,
+            mode=mode,
+            save_best_only=save_best_only,
+            save_weights_only=save_weights_only,
+            save_freq=save_freq,
+        )
+        self.trainer = None
+
+    def save_checkpoint(self, filepath: str):
+        # Note: `trainer.save_checkpoint` implementation contains invocation of
+        # `self.strategy.barrier("Trainer.save_checkpoint")`,
+        # in DDP training, this callback is only invoked in rank 0 process,
+        # the `barrier` invocation causes deadlock,
+        # so I implement `save_checkpoint` instead of
+        # calling `trainer.save_checkpoint`.
+        checkpoint = self.trainer._checkpoint_connector.dump_checkpoint(self.save_weights_only)
+        self.trainer.strategy.save_checkpoint(checkpoint, filepath)
+
+    @rank_zero_only
+    def on_fit_start(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        self.trainer = trainer
+
+    @rank_zero_only
+    def on_train_batch_end(
+        self,
+        trainer: "pl.Trainer",
+        pl_module: "pl.LightningModule",
+        outputs,
+        batch,
+        batch_idx,
+    ) -> None:
+        if isinstance(self.save_freq, int) and (
+            trainer.global_step > 0 and trainer.global_step % self.save_freq == 0
+        ):
+            self.check_and_save_checkpoint_if_needed(
+                current_epoch=trainer.current_epoch,
+                global_step=trainer.global_step,
+                metric_dict={k: float(v) for k, v in trainer.callback_metrics.items()},
+            )
+
+    @rank_zero_only
+    def on_train_epoch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        if self.save_freq == "epoch":
+            self.check_and_save_checkpoint_if_needed(
+                current_epoch=trainer.current_epoch,
+                global_step=trainer.global_step,
+                metric_dict={k: float(v) for k, v in trainer.callback_metrics.items()},
+            )
+
+
 # PyTorch-Lightning refactored the LoggerConnector class in version 1.4.0 and made metrics
 # update on demand. Prior to this, the metrics from the current step were not available to
 # callbacks immediately, so the view of metrics was off by one step.
@@ -294,9 +405,10 @@ def _log_early_stop_params(early_stop_callback, client, run_id):
     """
     Logs early stopping configuration parameters to MLflow.
 
-    :param early_stop_callback: The early stopping callback instance used during training.
-    :param client: An `MlflowAutologgingQueueingClient` instance used for MLflow logging.
-    :param run_id: The ID of the MLflow Run to which to log configuration parameters.
+    Args:
+        early_stop_callback: The early stopping callback instance used during training.
+        client: An `MlflowAutologgingQueueingClient` instance used for MLflow logging.
+        run_id: The ID of the MLflow Run to which to log configuration parameters.
     """
     client.log_params(
         run_id,
@@ -312,9 +424,10 @@ def _log_early_stop_metrics(early_stop_callback, client, run_id):
     """
     Logs early stopping behavior results (e.g. stopped epoch) as metrics to MLflow.
 
-    :param early_stop_callback: The early stopping callback instance used during training.
-    :param client: An `MlflowAutologgingQueueingClient` instance used for MLflow logging.
-    :param run_id: The ID of the MLflow Run to which to log configuration parameters.
+    Args:
+        early_stop_callback: The early stopping callback instance used during training.
+        client: An `MlflowAutologgingQueueingClient` instance used for MLflow logging.
+        run_id: The ID of the MLflow Run to which to log configuration parameters.
     """
     if early_stop_callback.stopped_epoch == 0:
         return
@@ -350,15 +463,12 @@ def patched_fit(original, self, *args, **kwargs):
     """
     if not MIN_REQ_VERSION <= _pl_version <= MAX_REQ_VERSION:
         warnings.warn(
-            (
-                "Autologging is known to be compatible with pytorch-lightning versions between "
-                "%s and %s and may not succeed with packages "
-                "outside this range."
-            )
-            % (MIN_REQ_VERSION, MAX_REQ_VERSION)
+            "Autologging is known to be compatible with pytorch-lightning versions between "
+            f"{MIN_REQ_VERSION} and {MAX_REQ_VERSION} and may not succeed with packages "
+            "outside this range."
         )
 
-    with _pytorch_autolog.disable_pytorch_autologging():
+    with disable_autologging():
         run_id = mlflow.active_run().info.run_id
         tracking_uri = mlflow.get_tracking_uri()
         client = MlflowAutologgingQueueingClient(tracking_uri)
@@ -378,13 +488,52 @@ def patched_fit(original, self, *args, **kwargs):
                 early_stop_callback = callback
                 _log_early_stop_params(early_stop_callback, client, run_id)
 
-        if not any(isinstance(callbacks, __MLflowPLCallback) for callbacks in self.callbacks):
+        if not any(isinstance(callbacks, __MlflowPLCallback) for callbacks in self.callbacks):
             self.callbacks += [
-                __MLflowPLCallback(
+                __MlflowPLCallback(
                     client, metrics_logger, run_id, log_models, log_every_n_epoch, log_every_n_step
                 )
             ]
 
+        model_checkpoint = get_autologging_config(mlflow.pytorch.FLAVOR_NAME, "checkpoint", True)
+        if model_checkpoint:
+            # __MLflowModelCheckpoint only supports pytorch-lightning >= 1.6.0
+            if _pl_version >= Version("1.6.0"):
+                checkpoint_monitor = get_autologging_config(
+                    mlflow.pytorch.FLAVOR_NAME, "checkpoint_monitor", "val_loss"
+                )
+                checkpoint_mode = get_autologging_config(
+                    mlflow.pytorch.FLAVOR_NAME, "checkpoint_mode", "min"
+                )
+                checkpoint_save_best_only = get_autologging_config(
+                    mlflow.pytorch.FLAVOR_NAME, "checkpoint_save_best_only", True
+                )
+                checkpoint_save_weights_only = get_autologging_config(
+                    mlflow.pytorch.FLAVOR_NAME, "checkpoint_save_weights_only", False
+                )
+                checkpoint_save_freq = get_autologging_config(
+                    mlflow.pytorch.FLAVOR_NAME, "checkpoint_save_freq", "epoch"
+                )
+
+                if not any(
+                    isinstance(callbacks, MlflowModelCheckpointCallback)
+                    for callbacks in self.callbacks
+                ):
+                    self.callbacks += [
+                        MlflowModelCheckpointCallback(
+                            monitor=checkpoint_monitor,
+                            mode=checkpoint_mode,
+                            save_best_only=checkpoint_save_best_only,
+                            save_weights_only=checkpoint_save_weights_only,
+                            save_freq=checkpoint_save_freq,
+                        )
+                    ]
+            else:
+                warnings.warn(
+                    "Automatic model checkpointing is disabled because this feature only "
+                    "supports pytorch-lightning >= 1.6.0."
+                )
+
         client.flush(synchronous=False)
 
         result = original(self, *args, **kwargs)
@@ -393,28 +542,24 @@ def patched_fit(original, self, *args, **kwargs):
             _log_early_stop_metrics(early_stop_callback, client, run_id)
 
         if Version(pl.__version__) < Version("1.4.0"):
-            # pylint: disable-next=unexpected-keyword-arg
             summary = str(ModelSummary(self.model, mode="full"))
         else:
             summary = str(ModelSummary(self.model, max_depth=-1))
 
-        tempdir = tempfile.mkdtemp()
-        try:
+        with tempfile.TemporaryDirectory() as tempdir:
             summary_file = os.path.join(tempdir, "model_summary.txt")
             with open(summary_file, "w") as f:
                 f.write(summary)
 
             mlflow.log_artifact(local_path=summary_file)
-        finally:
-            shutil.rmtree(tempdir)
 
         if log_models:
             registered_model_name = get_autologging_config(
                 mlflow.pytorch.FLAVOR_NAME, "registered_model_name", None
             )
             mlflow.pytorch.log_model(
-                pytorch_model=self.model,
-                artifact_path="model",
+                self.model,
+                "model",
                 registered_model_name=registered_model_name,
             )
 
