@@ -1,11 +1,13 @@
 import abc
 import logging
 import os
+from typing import Optional
 
 from mlflow.exceptions import MlflowException
+from mlflow.protos.databricks_pb2 import BAD_REQUEST, INTERNAL_ERROR, INVALID_PARAMETER_VALUE
 from mlflow.recipes import dag_help_strings
 from mlflow.recipes.artifacts import Artifact
-from mlflow.recipes.step import BaseStep, StepStatus, StepClass
+from mlflow.recipes.step import BaseStep, StepClass, StepStatus
 from mlflow.recipes.utils import (
     get_recipe_config,
     get_recipe_name,
@@ -13,36 +15,31 @@ from mlflow.recipes.utils import (
 )
 from mlflow.recipes.utils.execution import (
     clean_execution_state,
-    run_recipe_step,
     get_or_create_base_execution_directory,
     get_step_output_path,
+    run_recipe_step,
 )
 from mlflow.recipes.utils.step import display_html
-from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE, INTERNAL_ERROR, BAD_REQUEST
-from mlflow.utils.annotations import experimental
 from mlflow.utils.class_utils import _get_class_from_string
-from typing import List, Union
 
 _logger = logging.getLogger(__name__)
 
 
-@experimental
-class _BaseRecipe:
+class BaseRecipe:
     """
     Base Recipe
     """
 
-    @experimental
     def __init__(self, recipe_root_path: str, profile: str) -> None:
         """
         Recipe base class.
 
-        :param recipe_root_path: String path to the directory under which the recipe template
-                                   such as recipe.yaml, profiles/{profile}.yaml and
-                                   steps/{step_name}.py are defined.
-        :param profile: String specifying the profile name, with which
-                        {recipe_root_path}/profiles/{profile}.yaml is read and merged with
-                        recipe.yaml to generate the configuration to run the recipe.
+        Args:
+            recipe_root_path: String path to the directory under which the recipe template
+                such as recipe.yaml, profiles/{profile}.yaml and steps/{step_name}.py are defined.
+            profile: String specifying the profile name, with which
+                {recipe_root_path}/profiles/{profile}.yaml is read and merged with
+                recipe.yaml to generate the configuration to run the recipe.
         """
         self._recipe_root_path = recipe_root_path
         self._run_args = {}
@@ -55,13 +52,11 @@ class _BaseRecipe:
         self._steps = self._resolve_recipe_steps()
         self._recipe = get_recipe_config(self._recipe_root_path, self._profile).get("recipe")
 
-    @experimental
     @property
     def name(self) -> str:
         """Returns the name of the recipe."""
         return self._name
 
-    @experimental
     @property
     def profile(self) -> str:
         """
@@ -69,15 +64,17 @@ class _BaseRecipe:
         """
         return self._profile
 
-    @experimental
-    def run(self, step: str = None) -> None:
+    def run(self, step: Optional[str] = None) -> None:
         """
         Runs a step in the recipe, or the entire recipe if a step is not specified.
 
-        :param step: String name to run a step within the recipe. The step and its dependencies
-                     will be run sequentially. If a step is not specified, the entire recipe is
-                     executed.
-        :return: None
+        Args:
+            step: String name to run a step within the recipe. The step and its dependencies
+                will be run sequentially. If a step is not specified, the entire recipe is
+                executed.
+
+        Returns:
+            None
         """
 
         # TODO Record performance here.
@@ -100,29 +97,32 @@ class _BaseRecipe:
             last_executed_step_output_directory
         )
         if last_executed_step_state.status != StepStatus.SUCCEEDED:
+            last_step_error_mesg = (
+                f"The following error occurred while running step '{last_executed_step}':\n"
+                f"{last_executed_step_state.stack_trace}\n"
+                f"Last step status: '{last_executed_step_state.status}'\n"
+            )
             if step is not None:
                 raise MlflowException(
-                    f"Failed to run step '{step}' of recipe '{self.name}'."
-                    f" An error was encountered while running step '{last_executed_step.name}':"
-                    f" {last_executed_step_state.stack_trace}",
+                    f"Failed to run step '{step}' of recipe '{self.name}':\n{last_step_error_mesg}",
                     error_code=BAD_REQUEST,
                 )
             else:
                 raise MlflowException(
-                    f"Failed to run recipe '{self.name}'."
-                    f" An error was encountered while running step '{last_executed_step.name}':"
-                    f" {last_executed_step_state.stack_trace}",
+                    f"Failed to run recipe '{self.name}':\n{last_step_error_mesg}",
                     error_code=BAD_REQUEST,
                 )
 
-    @experimental
-    def inspect(self, step: str = None) -> None:
+    def inspect(self, step: Optional[str] = None) -> None:
         """
         Displays main output from a step, or a recipe DAG if no step is specified.
 
-        :param step: String name to display a step output within the recipe. If a step is not
-                     specified, the DAG of the recipe is shown instead.
-        :return: None
+        Args:
+            step: String name to display a step output within the recipe. If a step is not
+                specified, the DAG of the recipe is shown instead.
+
+        Returns:
+            None
         """
         self._steps = self._resolve_recipe_steps()
         if not step:
@@ -131,16 +131,16 @@ class _BaseRecipe:
             output_directory = get_step_output_path(self._recipe_root_path, step, "")
             self._get_step(step).inspect(output_directory)
 
-    @experimental
-    def clean(self, step: str = None) -> None:
+    def clean(self, step: Optional[str] = None) -> None:
         """
         Removes the outputs of the specified step from the cache, or removes the cached outputs
         of all steps if no particular step is specified. After cached outputs are cleaned
         for a particular step, the step will be re-executed in its entirety the next time it is
-        invoked via ``_BaseRecipe.run()``.
+        invoked via ``BaseRecipe.run()``.
 
-        :param step: String name of the step to clean within the recipe. If not specified,
-                     cached outputs are removed for all recipe steps.
+        Args:
+            step: String name of the step to clean within the recipe. If not specified,
+                cached outputs are removed for all recipe steps.
         """
         to_clean = self._steps if not step else [self._get_step(step)]
         clean_execution_state(self._recipe_root_path, to_clean)
@@ -155,8 +155,7 @@ class _BaseRecipe:
             )
         return self._steps[step_names.index(step_name)]
 
-    @experimental
-    def _get_subgraph_for_target_step(self, target_step: BaseStep) -> List[BaseStep]:
+    def _get_subgraph_for_target_step(self, target_step: BaseStep) -> list[BaseStep]:
         """
         Return a list of step objects representing a connected DAG containing the target_step.
         The returned list should be a sublist of self._steps.
@@ -169,7 +168,6 @@ class _BaseRecipe:
                 subgraph.append(step)
         return subgraph
 
-    @experimental
     @abc.abstractmethod
     def _get_default_step(self) -> BaseStep:
         """
@@ -177,9 +175,7 @@ class _BaseRecipe:
 
         Concrete recipe class should implement this method.
         """
-        pass
 
-    @experimental
     @abc.abstractmethod
     def _get_step_classes(self):
         """
@@ -187,9 +183,7 @@ class _BaseRecipe:
 
         Concrete recipe class should implement this method.
         """
-        pass
 
-    @experimental
     def _get_recipe_dag_file(self) -> str:
         """
         Returns absolute path to the recipe DAG representation HTML file.
@@ -318,7 +312,7 @@ class _BaseRecipe:
 
         return recipe_dag_file
 
-    def _resolve_recipe_steps(self) -> List[BaseStep]:
+    def _resolve_recipe_steps(self) -> list[BaseStep]:
         """
         Constructs and returns all recipe step objects from the recipe configuration.
         """
@@ -329,7 +323,6 @@ class _BaseRecipe:
             for s in self._get_step_classes()
         ]
 
-    @experimental
     def get_artifact(self, artifact_name: str):
         """
         Read an artifact from recipe output. artifact names can be obtained from
@@ -340,7 +333,6 @@ class _BaseRecipe:
         """
         return self._get_artifact(artifact_name).load()
 
-    @experimental
     def _get_artifact(self, artifact_name: str) -> Artifact:
         """
         Read an Artifact object from recipe output. artifact names can be obtained
@@ -359,11 +351,6 @@ class _BaseRecipe:
         )
 
 
-from mlflow.recipes.classification.v1.recipe import ClassificationRecipe
-from mlflow.recipes.regression.v1.recipe import RegressionRecipe
-
-
-@experimental
 class Recipe:
     """
     A factory class that creates an instance of a recipe for a particular ML problem
@@ -381,25 +368,25 @@ class Recipe:
         regression_recipe.run(step="train")
     """
 
-    @experimental
-    def __new__(cls, profile: str) -> Union[RegressionRecipe, ClassificationRecipe]:
+    def __new__(cls, profile: str):
         """
         Creates an instance of an MLflow Recipe for a particular ML problem or MLOps task based
         on the current working directory and supplied configuration. The current working directory
         must be the root directory of an MLflow Recipe repository or a subdirectory of an
         MLflow Recipe repository.
 
-        :param profile: The name of the profile to use for configuring the problem-specific or
-                        task-specific recipe. Profiles customize the configuration of
-                        one or more recipe steps, and recipe executions with different profiles
-                        often produce different results.
-        :return: A recipe for a particular ML problem or MLOps task. For example, an instance of
-                 :py:class:`RegressionRecipe
-                 <mlflow.recipes.regression.v1.recipe.RegressionRecipe>`
-                 for regression problems.
+        Args:
+            profile: The name of the profile to use for configuring the problem-specific or
+                task-specific recipe. Profiles customize the configuration of
+                one or more recipe steps, and recipe executions with different profiles
+                often produce different results.
+
+        Returns:
+            A recipe for a particular ML problem or MLOps task. For example, an instance of
+            `RegressionRecipe <https://github.com/mlflow/recipes-regression-template>`_
+            for regression problems.
 
         .. code-block:: python
-            :caption: Example
 
             import os
             from mlflow.recipes import Recipe
@@ -446,9 +433,9 @@ class Recipe:
                 ) from None
             else:
                 raise MlflowException(
-                    f"Failed to construct Recipe {class_name}. Error: {repr(e)}",
+                    f"Failed to construct Recipe {class_name}",
                     error_code=INTERNAL_ERROR,
-                ) from None
+                ) from e
 
         recipe_name = get_recipe_name(recipe_root_path)
         _logger.info(f"Creating MLflow Recipe '{recipe_name}' with profile: '{profile}'")
