@@ -4,16 +4,17 @@ import os
 import pathlib
 import re
 import shutil
-from typing import List, Dict
 
+from mlflow.environment_variables import (
+    MLFLOW_RECIPES_EXECUTION_DIRECTORY,
+    MLFLOW_RECIPES_EXECUTION_TARGET_STEP_NAME,
+)
 from mlflow.recipes.step import BaseStep, StepStatus
 from mlflow.utils.file_utils import read_yaml, write_yaml
 from mlflow.utils.process import _exec_cmd
 
 _logger = logging.getLogger(__name__)
 
-_MLFLOW_RECIPES_EXECUTION_DIRECTORY_ENV_VAR = "MLFLOW_RECIPES_EXECUTION_DIRECTORY"
-_MLFLOW_RECIPES_EXECUTION_TARGET_STEP_NAME_ENV_VAR = "MLFLOW_RECIPES_EXECUTION_TARGET_STEP_NAME"
 _STEPS_SUBDIRECTORY_NAME = "steps"
 _STEP_OUTPUTS_SUBDIRECTORY_NAME = "outputs"
 _STEP_CONF_YAML_NAME = "conf.yaml"
@@ -21,24 +22,27 @@ _STEP_CONF_YAML_NAME = "conf.yaml"
 
 def run_recipe_step(
     recipe_root_path: str,
-    recipe_steps: List[BaseStep],
+    recipe_steps: list[BaseStep],
     target_step: BaseStep,
     template: str,
 ) -> BaseStep:
     """
     Runs the specified step in the specified recipe, as well as all dependent steps.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :param recipe_steps: A list of all the steps contained in the subgraph of the specified
-                           recipe that contains the target_step. Recipe steps must be
-                           provided in the order that they are intended to be executed.
-    :param target_step: The step to run.
-    :param template: The template to use when selecting a Makefile to load.  If the template is
-                     invalid, an exception is thrown.
-    :return: The last step that successfully completed during the recipe execution. If execution
-             was successful, this always corresponds to the supplied target step. If execution was
-             unsuccessful, this corresponds to the step that failed.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+        recipe_steps: A list of all the steps contained in the subgraph of the specified
+            recipe that contains the target_step. Recipe steps must be provided in the order
+            that they are intended to be executed.
+        target_step: The step to run.
+        template: The template to use when selecting a Makefile to load. If the template is
+            invalid, an exception is thrown.
+
+    Returns:
+        The last step that successfully completed during the recipe execution. If execution
+        was successful, this always corresponds to the supplied target step. If execution was
+        unsuccessful, this corresponds to the step that failed.
     """
     target_step_index = recipe_steps.index(target_step)
     execution_dir_path = _get_or_create_execution_directory(
@@ -75,7 +79,7 @@ def run_recipe_step(
     # should be isolated in different subprocesses
     make_env = {
         # Include target step name in the environment variable set
-        _MLFLOW_RECIPES_EXECUTION_TARGET_STEP_NAME_ENV_VAR: target_step.name,
+        MLFLOW_RECIPES_EXECUTION_TARGET_STEP_NAME.name: target_step.name,
     }
     for step in recipe_steps:
         make_env.update(step.environment)
@@ -113,15 +117,16 @@ def run_recipe_step(
     return last_executed_step
 
 
-def clean_execution_state(recipe_root_path: str, recipe_steps: List[BaseStep]) -> None:
+def clean_execution_state(recipe_root_path: str, recipe_steps: list[BaseStep]) -> None:
     """
     Removes all execution state for the specified recipe steps from the associated execution
     directory on the local filesystem. This method does *not* remove other execution results, such
     as content logged to MLflow Tracking.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :param recipe_steps: The recipe steps for which to remove execution state.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+        recipe_steps: The recipe steps for which to remove execution state.
     """
     execution_dir_path = get_or_create_base_execution_directory(recipe_root_path=recipe_root_path)
     for step in recipe_steps:
@@ -139,13 +144,16 @@ def get_step_output_path(recipe_root_path: str, step_name: str, relative_path: s
     Obtains the absolute path of the specified step output on the local filesystem. Does
     not check the existence of the output.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :param step_name: The name of the recipe step containing the specified output.
-    :param relative_path: The relative path of the output within the output directory
-                          of the specified recipe step.
-    :return The absolute path of the step output on the local filesystem, which may or may
-            not exist.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+        step_name: The name of the recipe step containing the specified output.
+        relative_path: The relative path of the output within the output directory
+            of the specified recipe step.
+
+    Returns:
+        The absolute path of the step output on the local filesystem, which may or may
+        not exist.
     """
     execution_dir_path = get_or_create_base_execution_directory(recipe_root_path=recipe_root_path)
     step_outputs_path = _get_step_output_directory_path(
@@ -156,19 +164,22 @@ def get_step_output_path(recipe_root_path: str, step_name: str, relative_path: s
 
 
 def _get_or_create_execution_directory(
-    recipe_root_path: str, recipe_steps: List[BaseStep], template: str
+    recipe_root_path: str, recipe_steps: list[BaseStep], template: str
 ) -> str:
     """
     Obtains the path of the execution directory on the local filesystem corresponding to the
     specified recipe, creating the execution directory and its required contents if they do
     not already exist.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :param recipe_steps: A list of all the steps contained in the specified recipe.
-    :param template: The template to use to generate the makefile.
-    :return: The absolute path of the execution directory on the local filesystem for the specified
-             recipe.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+        recipe_steps: A list of all the steps contained in the specified recipe.
+        template: The template to use to generate the makefile.
+
+    Returns:
+        The absolute path of the execution directory on the local filesystem for the specified
+        recipe.
     """
     execution_dir_path = get_or_create_base_execution_directory(recipe_root_path=recipe_root_path)
 
@@ -180,18 +191,18 @@ def _get_or_create_execution_directory(
     return execution_dir_path
 
 
-def _write_updated_step_confs(recipe_steps: List[BaseStep], execution_directory_path: str) -> None:
+def _write_updated_step_confs(recipe_steps: list[BaseStep], execution_directory_path: str) -> None:
     """
     Compares the in-memory configuration state of the specified recipe steps with step-specific
     internal configuration files written by prior executions. If updates are found, writes updated
     state to the corresponding files. If no updates are found, configuration state is not
     rewritten.
 
-    :param recipe_steps: A list of all the steps contained in the specified recipe.
-    :param execution_directory_path: The absolute path of the execution directory on the local
-                                     filesystem for the specified recipe. Configuration files are
-                                     written to step-specific subdirectories of this execution
-                                     directory.
+    Args:
+        recipe_steps: A list of all the steps contained in the specified recipe.
+        execution_directory_path: The absolute path of the execution directory on the local
+            filesystem for the specified recipe. Configuration files are written to step-specific
+            subdirectories of this execution directory.
     """
     for step in recipe_steps:
         step_subdir_path = os.path.join(
@@ -218,17 +229,20 @@ def get_or_create_base_execution_directory(recipe_root_path: str) -> str:
     Obtains the path of the execution directory on the local filesystem corresponding to the
     specified recipe. The directory is created if it does not exist.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :return: The path of the execution directory on the local filesystem corresponding to the
-             specified recipe.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+
+    Returns:
+        The path of the execution directory on the local filesystem corresponding to the
+        specified recipe.
     """
     execution_directory_basename = _get_execution_directory_basename(
         recipe_root_path=recipe_root_path
     )
 
     execution_dir_path = os.path.abspath(
-        os.environ.get(_MLFLOW_RECIPES_EXECUTION_DIRECTORY_ENV_VAR)
+        MLFLOW_RECIPES_EXECUTION_DIRECTORY.get()
         or os.path.join(os.path.expanduser("~"), ".mlflow", "recipes", execution_directory_basename)
     )
     os.makedirs(execution_dir_path, exist_ok=True)
@@ -239,9 +253,12 @@ def _get_execution_directory_basename(recipe_root_path):
     """
     Obtains the basename of the execution directory corresponding to the specified recipe.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :return: The basename of the execution directory corresponding to the specified recipe.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+
+    Returns:
+        The basename of the execution directory corresponding to the specified recipe.
     """
     return hashlib.sha256(os.path.abspath(recipe_root_path).encode("utf-8")).hexdigest()
 
@@ -251,12 +268,14 @@ def _get_step_output_directory_path(execution_directory_path: str, step_name: st
     Obtains the path of the local filesystem directory containing outputs for the specified step,
     which may or may not exist.
 
-    :param execution_directory_path: The absolute path of the execution directory on the local
-                                     filesystem for the relevant recipe. The Makefile is created
-                                     in this directory.
-    :param step_name: The name of the recipe step for which to obtain the output directory path.
-    :return The absolute path of the local filesystem directory containing outputs for the specified
-            step.
+    Args:
+        execution_directory_path: The absolute path of the execution directory on the local
+            filesystem for the relevant recipe. The Makefile is created in this directory.
+        step_name: The name of the recipe step for which to obtain the output directory path.
+
+    Returns:
+        The absolute path of the local filesystem directory containing outputs for the specified
+        step.
     """
     return os.path.abspath(
         os.path.join(
@@ -269,15 +288,15 @@ def _get_step_output_directory_path(execution_directory_path: str, step_name: st
 
 
 class _ExecutionPlan:
-    _MSG_REGEX = r'^echo "Run MLFlow Recipe step: (\w+)"\n$'
+    _MSG_REGEX = r'^echo "Run MLflow Recipe step: (\w+)"\n$'
     _FORMAT_STEPS_CACHED = "%s: No changes. Skipping."
 
-    def __init__(self, rule_name, output_lines_of_make: List[str], recipe_step_names: List[str]):
+    def __init__(self, rule_name, output_lines_of_make: list[str], recipe_step_names: list[str]):
         steps_to_run = self._parse_output_lines(output_lines_of_make)
         self.steps_cached = self._infer_cached_steps(rule_name, steps_to_run, recipe_step_names)
 
     @staticmethod
-    def _parse_output_lines(output_lines_of_make: List[str]) -> List[str]:
+    def _parse_output_lines(output_lines_of_make: list[str]) -> list[str]:
         """
         Parse the output lines of Make to get steps to run.
         """
@@ -295,14 +314,17 @@ class _ExecutionPlan:
         return list(steps_to_run())
 
     @staticmethod
-    def _infer_cached_steps(rule_name, steps_to_run, recipe_step_names) -> List[str]:
+    def _infer_cached_steps(rule_name, steps_to_run, recipe_step_names) -> list[str]:
         """
         Infer cached steps.
-        :param rule_name: The name of the Make rule to run.
-        :param steps_to_run: The step names obtained by parsing the Make output showing
-                             which steps will be executed.
-        :param recipe_step_names:  A list of all the step names contained in the specified
-                                     recipe sorted by the execution order.
+
+        Args:
+            rule_name: The name of the Make rule to run.
+            steps_to_run: The step names obtained by parsing the Make output showing
+                which steps will be executed.
+            recipe_step_names: A list of all the step names contained in the specified
+                recipe sorted by the execution order.
+
         """
         index = recipe_step_names.index(rule_name)
         if index == 0:
@@ -325,20 +347,20 @@ class _ExecutionPlan:
 def _run_make(
     execution_directory_path,
     rule_name: str,
-    extra_env: Dict[str, str],
-    recipe_steps: List[BaseStep],
+    extra_env: dict[str, str],
+    recipe_steps: list[BaseStep],
 ) -> None:
     """
     Runs the specified recipe rule with Make. This method assumes that a Makefile named `Makefile`
     exists in the specified execution directory.
 
-    :param execution_directory_path: The absolute path of the execution directory on the local
-                                     filesystem for the relevant recipe. The Makefile is created
-                                     in this directory.
-    :param extra_env: Extra environment variables to be defined when running the Make child process.
-    :param rule_name: The name of the Make rule to run.
-    :param recipe_steps: A list of step instances that is a subgraph containing the step specified
-                           by `rule_name`.
+    Args:
+        execution_directory_path: The absolute path of the execution directory on the local
+            filesystem for the relevant recipe. The Makefile is created in this directory.
+        rule_name: The name of the Make rule to run.
+        extra_env: Extra environment variables to be defined when running the Make child process.
+        recipe_steps: A list of step instances that is a subgraph containing the step specified
+            by `rule_name`.
     """
     # Dry-run Make and collect the outputs
     process = _exec_cmd(
@@ -378,12 +400,12 @@ def _create_makefile(recipe_root_path, execution_directory_path, template) -> No
     overwriting the preexisting Makefile if one exists. The Makefile is created in the specified
     execution directory.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :param execution_directory_path: The absolute path of the execution directory on the local
-                                     filesystem for the specified recipe. The Makefile is created
-                                     in this directory.
-    :param template: The template to use to generate the makefile.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+        execution_directory_path: The absolute path of the execution directory on the local
+            filesystem for the specified recipe. The Makefile is created in this directory.
+        template: The template to use to generate the makefile.
     """
     makefile_path = os.path.join(execution_directory_path, "Makefile")
 
@@ -403,7 +425,7 @@ def _create_makefile(recipe_root_path, execution_directory_path, template) -> No
             if not os.path.exists(required_file_path):
                 try:
                     with open(required_file_path, "w") as f:
-                        f.write("# Created by MLflow Pipeliens\n")
+                        f.write("# Created by MLflow Pipelines\n")
                 except OSError:
                     pass
             if not os.path.exists(required_file_path):
@@ -449,10 +471,11 @@ class _MakefilePathFormat:
 
     def __init__(self, recipe_root_path: str, execution_directory_path: str):
         """
-        :param recipe_root_path: The absolute path of the recipe root directory on the local
-                                   filesystem.
-        :param execution_directory_path: The absolute path of the execution directory on the local
-                                         filesystem for the recipe.
+        Args:
+            recipe_root_path: The absolute path of the recipe root directory on the local
+                filesystem.
+            execution_directory_path: The absolute path of the execution directory on the local
+                filesystem for the recipe.
         """
         self.recipe_root_path = recipe_root_path
         self.execution_directory_path = execution_directory_path
@@ -461,16 +484,19 @@ class _MakefilePathFormat:
         self, path_spec: str, prefix_placeholder: str, replacement_path: str
     ) -> str:
         """
-        :param path_spec: A substitution path spec of the form `<placeholder>/<subpath>`. This
-                          method substitutes `<placeholder>` with `<recipe_root_path>`, if
-                          `<placeholder>` is `prp`, or `<execution_directory_path>`, if
-                          `<placeholder>` is `exe`.
-        :param prefix_placeholder: The prefix placeholder, which is present at the beginning of
-                                   `path_spec`. Either `prp` or `exe`.
-        :param replacement_path: The path to use to replace the specified `prefix_placeholder`
-                                 in the specified `path_spec`.
-        :return: The formatted path obtained by replacing the ``prefix placeholder`` in the
-                 specified ``path_spec`` with the specified ``replacement_path``.
+        Args:
+            path_spec: A substitution path spec of the form `<placeholder>/<subpath>`. This
+                method substitutes `<placeholder>` with `<recipe_root_path>`, if
+                `<placeholder>` is `prp`, or `<execution_directory_path>`, if
+                `<placeholder>` is `exe`.
+            prefix_placeholder: The prefix placeholder, which is present at the beginning of
+                `path_spec`. Either `prp` or `exe`.
+            replacement_path: The path to use to replace the specified `prefix_placeholder`
+                in the specified `path_spec`.
+
+        Returns:
+            The formatted path obtained by replacing the ``prefix placeholder`` in the
+            specified ``path_spec`` with the specified ``replacement_path``.
         """
         subpath = pathlib.PurePosixPath(path_spec.split(prefix_placeholder)[1])
         recipe_root_posix_path = pathlib.PurePosixPath(pathlib.Path(replacement_path).as_posix())
@@ -479,10 +505,11 @@ class _MakefilePathFormat:
 
     def __format__(self, path_spec: str) -> str:
         """
-        :param path_spec: A substitution path spec of the form `<placeholder>/<subpath>`. This
-                          method substitutes `<placeholder>` with `<recipe_root_path>`, if
-                          `<placeholder>` is `prp`, or `<execution_directory_path>`, if
-                          `<placeholder>` is `exe`.
+        Args:
+            path_spec: A substitution path spec of the form `<placeholder>/<subpath>`. This
+                method substitutes `<placeholder>` with `<recipe_root_path>`, if
+                `<placeholder>` is `prp`, or `<execution_directory_path>`, if
+                `<placeholder>` is `exe`.
         """
         if path_spec.startswith(_MakefilePathFormat._RECIPE_ROOT_PATH_PREFIX_PLACEHOLDER):
             return self._get_formatted_path(
@@ -515,7 +542,7 @@ ingest:
 # ensuring that data is only ingested for downstream steps if it is not already present on the
 # local filesystem
 steps/ingest/outputs/dataset.parquet: steps/ingest/conf.yaml {path:prp/steps/ingest.py}
-	echo "Run MLFlow Recipe step: ingest"
+	echo "Run MLflow Recipe step: ingest"
 	$(MAKE) ingest
 
 split_objects = steps/split/outputs/train.parquet steps/split/outputs/validation.parquet steps/split/outputs/test.parquet
@@ -523,7 +550,7 @@ split_objects = steps/split/outputs/train.parquet steps/split/outputs/validation
 split: $(split_objects)
 
 steps/%/outputs/train.parquet steps/%/outputs/validation.parquet steps/%/outputs/test.parquet: {path:prp/steps/split.py} steps/ingest/outputs/dataset.parquet steps/split/conf.yaml
-	echo "Run MLFlow Recipe step: split"
+	echo "Run MLflow Recipe step: split"
 	cd {path:prp/} && \
         python -c "from mlflow.recipes.steps.split import SplitStep; SplitStep.from_step_config_path(step_config_path='{path:exe/steps/split/conf.yaml}', recipe_root='{path:prp/}').run(output_directory='{path:exe/steps/split/outputs}')"
 
@@ -532,7 +559,7 @@ transform_objects = steps/transform/outputs/transformer.pkl steps/transform/outp
 transform: $(transform_objects)
 
 steps/%/outputs/transformer.pkl steps/%/outputs/transformed_training_data.parquet steps/%/outputs/transformed_validation_data.parquet: {path:prp/steps/transform.py} steps/split/outputs/train.parquet steps/split/outputs/validation.parquet steps/transform/conf.yaml
-	echo "Run MLFlow Recipe step: transform"
+	echo "Run MLflow Recipe step: transform"
 	cd {path:prp/} && \
         python -c "from mlflow.recipes.steps.transform import TransformStep; TransformStep.from_step_config_path(step_config_path='{path:exe/steps/transform/conf.yaml}', recipe_root='{path:prp/}').run(output_directory='{path:exe/steps/transform/outputs}')"
 
@@ -541,7 +568,7 @@ train_objects = steps/train/outputs/model steps/train/outputs/run_id
 train: $(train_objects)
 
 steps/%/outputs/model steps/%/outputs/run_id: {path:prp/steps/train.py} {path:prp/steps/custom_metrics.py} steps/transform/outputs/transformed_training_data.parquet steps/transform/outputs/transformed_validation_data.parquet steps/split/outputs/train.parquet steps/split/outputs/validation.parquet steps/transform/outputs/transformer.pkl steps/train/conf.yaml
-	echo "Run MLFlow Recipe step: train"
+	echo "Run MLflow Recipe step: train"
 	cd {path:prp/} && \
         python -c "from mlflow.recipes.steps.train import TrainStep; TrainStep.from_step_config_path(step_config_path='{path:exe/steps/train/conf.yaml}', recipe_root='{path:prp/}').run(output_directory='{path:exe/steps/train/outputs}')"
 
@@ -550,7 +577,7 @@ evaluate_objects = steps/evaluate/outputs/model_validation_status
 evaluate: $(evaluate_objects)
 
 steps/%/outputs/model_validation_status: {path:prp/steps/custom_metrics.py} steps/train/outputs/model steps/split/outputs/validation.parquet steps/split/outputs/test.parquet steps/train/outputs/run_id steps/evaluate/conf.yaml
-	echo "Run MLFlow Recipe step: evaluate"
+	echo "Run MLflow Recipe step: evaluate"
 	cd {path:prp/} && \
         python -c "from mlflow.recipes.steps.evaluate import EvaluateStep; EvaluateStep.from_step_config_path(step_config_path='{path:exe/steps/evaluate/conf.yaml}', recipe_root='{path:prp/}').run(output_directory='{path:exe/steps/evaluate/outputs}')"
 
@@ -559,7 +586,7 @@ register_objects = steps/register/outputs/registered_model_version.json
 register: $(register_objects)
 
 steps/%/outputs/registered_model_version.json: steps/train/outputs/run_id steps/register/conf.yaml steps/evaluate/outputs/model_validation_status
-	echo "Run MLFlow Recipe step: register"
+	echo "Run MLflow Recipe step: register"
 	cd {path:prp/} && \
         python -c "from mlflow.recipes.steps.register import RegisterStep; RegisterStep.from_step_config_path(step_config_path='{path:exe/steps/register/conf.yaml}', recipe_root='{path:prp/}').run(output_directory='{path:exe/steps/register/outputs}')"
 
@@ -574,7 +601,7 @@ ingest_scoring:
 # `ingest_scoring` target, ensuring that data is only ingested for downstream steps if it is not
 # already present on the local filesystem
 steps/ingest_scoring/outputs/scoring-dataset.parquet: steps/ingest_scoring/conf.yaml {path:prp/steps/ingest.py}
-	echo "Run MLFlow Recipe step: ingest_scoring"
+	echo "Run MLflow Recipe step: ingest_scoring"
 	$(MAKE) ingest_scoring
 
 predict_objects = steps/predict/outputs/scored.parquet
@@ -582,10 +609,10 @@ predict_objects = steps/predict/outputs/scored.parquet
 predict: $(predict_objects)
 
 steps/predict/outputs/scored.parquet: steps/ingest_scoring/outputs/scoring-dataset.parquet steps/predict/conf.yaml
-	echo "Run MLFlow Recipe step: predict"
+	echo "Run MLflow Recipe step: predict"
 	cd {path:prp/} && \
         python -c "from mlflow.recipes.steps.predict import PredictStep; PredictStep.from_step_config_path(step_config_path='{path:exe/steps/predict/conf.yaml}', recipe_root='{path:prp/}').run(output_directory='{path:exe/steps/predict/outputs}')"
 
 clean:
 	rm -rf $(split_objects) $(transform_objects) $(train_objects) $(evaluate_objects) $(predict_objects)
-"""
+"""  # noqa: E501
