@@ -1,35 +1,34 @@
 import json
 import logging
 import pathlib
-import tempfile
 import shutil
+import tempfile
 import uuid
-from typing import Dict, Any, TypeVar
+from typing import Any, Optional
 
 import mlflow
+from mlflow.environment_variables import MLFLOW_RUN_CONTEXT
 from mlflow.exceptions import MlflowException, RestException
-from mlflow.recipes.utils import get_recipe_name
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
+from mlflow.recipes.utils import get_recipe_name
 from mlflow.tracking.client import MlflowClient
 from mlflow.tracking.context.registry import resolve_tags
-from mlflow.tracking.context.system_environment_context import MLFLOW_RUN_CONTEXT_ENV_VAR
 from mlflow.tracking.default_experiment import DEFAULT_EXPERIMENT_ID
-from mlflow.tracking.fluent import set_experiment as fluent_set_experiment, _get_experiment_id
+from mlflow.tracking.fluent import _get_experiment_id
+from mlflow.tracking.fluent import set_experiment as fluent_set_experiment
 from mlflow.utils.databricks_utils import is_in_databricks_runtime
-from mlflow.utils.file_utils import path_to_local_sqlite_uri, path_to_local_file_uri
-from mlflow.utils.git_utils import get_git_repo_url, get_git_commit, get_git_branch
+from mlflow.utils.file_utils import path_to_local_file_uri, path_to_local_sqlite_uri
+from mlflow.utils.git_utils import get_git_branch, get_git_commit, get_git_repo_url
 from mlflow.utils.mlflow_tags import (
-    MLFLOW_SOURCE_NAME,
+    LEGACY_MLFLOW_GIT_REPO_URL,
     MLFLOW_GIT_BRANCH,
     MLFLOW_GIT_COMMIT,
     MLFLOW_GIT_REPO_URL,
-    LEGACY_MLFLOW_GIT_REPO_URL,
+    MLFLOW_SOURCE_NAME,
 )
 
 _logger = logging.getLogger(__name__)
 
-TrackingConfigType = TypeVar("TrackingConfig")
-
 
 def _get_run_name(run_name_prefix):
     if run_name_prefix is None:
@@ -55,26 +54,26 @@ class TrackingConfig:
     def __init__(
         self,
         tracking_uri: str,
-        experiment_name: str = None,
-        experiment_id: str = None,
-        run_name: str = None,
-        artifact_location: str = None,
+        experiment_name: Optional[str] = None,
+        experiment_id: Optional[str] = None,
+        run_name: Optional[str] = None,
+        artifact_location: Optional[str] = None,
     ):
         """
-        :param tracking_uri: The MLflow Tracking URI.
-        :param experiment_name: The MLflow Experiment name. At least one of ``experiment_name`` or
-                                ``experiment_id`` must be specified. If both are specified, they
-                                must be consistent with Tracking server state. Note that this
-                                Experiment may not exist prior to recipe execution.
-        :param experiment_id: The MLflow Experiment ID. At least one of ``experiment_name`` or
-                              ``experiment_id`` must be specified. If both are specified, they
-                              must be consistent with Tracking server state. Note that this
-                              Experiment may not exist prior to recipe execution.
-        :param run_name: The MLFlow Run Name. If the run name is not specified, then a random
-                                name is set for the run.
-        :param artifact_location: The artifact location to use for the Experiment, if the Experiment
-                                  does not already exist. If the Experiment already exists, this
-                                  location is ignored.
+        Args:
+            tracking_uri: The MLflow Tracking URI.
+            experiment_name: The MLflow Experiment name. At least one of ``experiment_name`` or
+                ``experiment_id`` must be specified. If both are specified, they must be consistent
+                with Tracking server state. Note that this Experiment may not exist prior to recipe
+                execution.
+            experiment_id: The MLflow Experiment ID. At least one of ``experiment_name`` or
+                ``experiment_id`` must be specified. If both are specified, they must be consistent
+                with Tracking server state. Note that this Experiment may not exist prior to recipe
+                execution.
+            run_name: The MLflow Run Name. If the run name is not specified, then a random name is
+                set for the run.
+            artifact_location: The artifact location to use for the Experiment, if the Experiment
+                does not already exist. If the Experiment already exists, this location is ignored.
         """
         if tracking_uri is None:
             raise MlflowException(
@@ -93,11 +92,12 @@ class TrackingConfig:
         self.run_name = run_name
         self.artifact_location = artifact_location
 
-    def to_dict(self) -> Dict[str, str]:
+    def to_dict(self) -> dict[str, str]:
         """
         Obtains a dictionary representation of the MLflow Tracking configuration.
 
-        :return: A dictionary representation of the MLflow Tracking configuration.
+        Returns:
+            A dictionary representation of the MLflow Tracking configuration.
         """
         config_dict = {
             TrackingConfig._KEY_TRACKING_URI: self.tracking_uri,
@@ -118,12 +118,15 @@ class TrackingConfig:
         return config_dict
 
     @classmethod
-    def from_dict(cls, config_dict: Dict[str, str]) -> TrackingConfigType:
+    def from_dict(cls, config_dict: dict[str, str]) -> "TrackingConfig":
         """
         Creates a ``TrackingConfig`` instance from a dictionary representation.
 
-        :param config_dict: A dictionary representation of the MLflow Tracking configuration.
-        :return: A ``TrackingConfig`` instance.
+        Args:
+            config_dict: A dictionary representation of the MLflow Tracking configuration.
+
+        Returns:
+            A ``TrackingConfig`` instance.
         """
         return TrackingConfig(
             tracking_uri=config_dict.get(TrackingConfig._KEY_TRACKING_URI),
@@ -135,16 +138,19 @@ class TrackingConfig:
 
 
 def get_recipe_tracking_config(
-    recipe_root_path: str, recipe_config: Dict[str, Any]
+    recipe_root_path: str, recipe_config: dict[str, Any]
 ) -> TrackingConfig:
     """
     Obtains the MLflow Tracking configuration for the specified recipe.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :param recipe_config: The configuration of the specified recipe.
-    :return: A ``TrackingConfig`` instance containing MLflow Tracking information for the
-             specified recipe, including Tracking URI, Experiment name, and more.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+        recipe_config: The configuration of the specified recipe.
+
+    Returns:
+        A ``TrackingConfig`` instance containing MLflow Tracking information for the
+        specified recipe, including Tracking URI, Experiment name, and more.
     """
     if is_in_databricks_runtime():
         default_tracking_uri = "databricks"
@@ -200,7 +206,8 @@ def apply_recipe_tracking_config(tracking_config: TrackingConfig):
     MLflow Tracking URI (via ``mlflow.set_tracking_uri()``) and setting the associated MLflow
     Experiment (via ``mlflow.set_experiment()``), creating it if necessary.
 
-    :param tracking_config: The MLflow Recipe ``TrackingConfig`` to apply.
+    Args:
+        tracking_config: The MLflow Recipe ``TrackingConfig`` to apply.
     """
     mlflow.set_tracking_uri(uri=tracking_config.tracking_uri)
 
@@ -223,7 +230,7 @@ def apply_recipe_tracking_config(tracking_config: TrackingConfig):
                 raise MlflowException(
                     f"Could not create an MLflow Experiment with "
                     f"name {tracking_config.experiment_name}. Please create an "
-                    f"MLflow Experiment for this recipe and specify its name in the"
+                    f"MLflow Experiment for this recipe and specify its name in the "
                     f'"name" field of the "experiment" section in your profile configuration.'
                 )
 
@@ -232,15 +239,18 @@ def apply_recipe_tracking_config(tracking_config: TrackingConfig):
     )
 
 
-def get_run_tags_env_vars(recipe_root_path: str) -> Dict[str, str]:
+def get_run_tags_env_vars(recipe_root_path: str) -> dict[str, str]:
     """
     Returns environment variables that should be set during step execution to ensure that MLflow
     Run Tags from the current context are applied to any MLflow Runs that are created during
     recipe execution.
 
-    :param recipe_root_path: The absolute path of the recipe root directory on the local
-                               filesystem.
-    :return: A dictionary of environment variable names and values.
+    Args:
+        recipe_root_path: The absolute path of the recipe root directory on the local
+            filesystem.
+
+    Returns:
+        A dictionary of environment variable names and values.
     """
     run_context_tags = resolve_tags()
 
@@ -257,22 +267,23 @@ def get_run_tags_env_vars(recipe_root_path: str) -> Dict[str, str]:
     if git_branch:
         git_tags[MLFLOW_GIT_BRANCH] = git_branch
 
-    return {MLFLOW_RUN_CONTEXT_ENV_VAR: json.dumps({**run_context_tags, **git_tags})}
+    return {MLFLOW_RUN_CONTEXT.name: json.dumps({**run_context_tags, **git_tags})}
 
 
 def log_code_snapshot(
     recipe_root: str,
     run_id: str,
     artifact_path: str = "recipe_snapshot",
-    recipe_config: Dict[str, Any] = None,
+    recipe_config: Optional[dict[str, Any]] = None,
 ) -> None:
     """
     Logs a recipe code snapshot as mlflow artifacts.
 
-    :param recipe_root_path: String file path to the directory where the recipe is defined.
-    :param run_id: Run ID to which the code snapshot is logged.
-    :param artifact_path: Directory within the run's artifact director (default: "snapshots").
-    :param recipe_config: Dict containing the full recipe configuration at runtime.
+    Args:
+        recipe_root: String file path to the directory where the recipe is defined.
+        run_id: Run ID to which the code snapshot is logged.
+        artifact_path: Directory within the run's artifact director (default: "snapshots").
+        recipe_config: Dict containing the full recipe configuration at runtime.
     """
     with tempfile.TemporaryDirectory() as tmpdir:
         tmpdir = pathlib.Path(tmpdir)
@@ -288,7 +299,7 @@ def log_code_snapshot(
             if file_path.exists():
                 tmp_path = tmpdir.joinpath(file_path.relative_to(recipe_root))
                 tmp_path.parent.mkdir(exist_ok=True, parents=True)
-                shutil.copyfile(file_path, tmp_path)
+                shutil.copy2(file_path, tmp_path)
         if recipe_config is not None:
             import yaml
 
