@@ -1,14 +1,15 @@
-import os
 import json
+import os
+import tempfile
 
 import click
 
 import mlflow
+import mlflow.models.docker_utils
 import mlflow.sagemaker
 from mlflow.sagemaker import DEFAULT_IMAGE_NAME as IMAGE
 from mlflow.utils import cli_args
 from mlflow.utils import env_manager as em
-import mlflow.models.docker_utils
 
 
 @click.group("sagemaker")
@@ -19,7 +20,6 @@ def commands():
     To serve a model associated with a run on a tracking server, set the MLFLOW_TRACKING_URI
     environment variable to the URL of the desired server.
     """
-    pass
 
 
 @commands.command("deploy-transform-job")
@@ -111,11 +111,9 @@ def commands():
     "-f",
     default=None,
     help=(
-        "The name of the flavor to use for deployment. Must be one of the following:"
-        " {supported_flavors}. If unspecified, a flavor will be automatically selected"
-        " from the model's available flavors.".format(
-            supported_flavors=mlflow.sagemaker.SUPPORTED_DEPLOYMENT_FLAVORS
-        )
+        "The name of the flavor to use for deployment. Must be one of the following: "
+        f"{mlflow.sagemaker.SUPPORTED_DEPLOYMENT_FLAVORS}. If unspecified, a flavor will be "
+        "automatically selected from the model's available flavors."
     ),
 )
 @click.option(
@@ -352,30 +350,35 @@ def build_and_push_container(build, push, container, env_manager, mlflow_home):
     The image is built locally and it requires Docker to run.
     The image is pushed to ECR under current active AWS account and to current active AWS region.
     """
+    from mlflow.models import docker_utils
+
     env_manager = env_manager or em.VIRTUALENV
     if not (build or push):
         click.echo("skipping both build and push, have nothing to do!")
     if build:
-        sagemaker_image_entrypoint = f"""
-        ENTRYPOINT ["python", "-c", "import sys; from mlflow.models import container as C; \
-        C._init(sys.argv[1], '{env_manager}')"]
-        """
+        sagemaker_image_entrypoint = (
+            "import sys; from mlflow.models import container as C; "
+            f"C._init(sys.argv[1], '{env_manager}')"
+        )
 
-        def setup_container(_):
-            return "\n".join(
-                [
-                    'ENV {disable_env}="false"',
-                    'RUN python -c "from mlflow.models.container import _install_pyfunc_deps;'
-                    '_install_pyfunc_deps(None, False)"',
-                ]
+        setup_container = (
+            "# Install minimal serving dependencies\n"
+            'RUN python -c "from mlflow.models.container import _install_pyfunc_deps;'
+            '_install_pyfunc_deps(None, False)"'
+        )
+
+        with tempfile.TemporaryDirectory() as tmp:
+            docker_utils.generate_dockerfile(
+                base_image=mlflow.models.docker_utils.UBUNTU_BASE_IMAGE,
+                output_dir=tmp,
+                entrypoint=sagemaker_image_entrypoint,
+                env_manager=env_manager,
+                mlflow_home=os.path.abspath(mlflow_home) if mlflow_home else None,
+                model_install_steps=setup_container,
+                # Create a conda env or virtualenv at runtime after the model is loaded
+                disable_env_creation_at_runtime=False,
             )
 
-        mlflow.models.docker_utils._build_image(
-            container,
-            mlflow_home=os.path.abspath(mlflow_home) if mlflow_home else None,
-            entrypoint=sagemaker_image_entrypoint,
-            custom_setup_steps_hook=setup_container,
-            env_manager=env_manager,
-        )
+            docker_utils.build_image_from_context(tmp, image_name=container)
     if push:
         mlflow.sagemaker.push_image_to_ecr(container)
