@@ -1,17 +1,22 @@
 import collections
 import inspect
 import logging
-import numpy as np
+import pkgutil
+import platform
 import warnings
 from copy import deepcopy
+from importlib import import_module
 from numbers import Number
+from operator import itemgetter
+
+import numpy as np
 from packaging.version import Version
 
 from mlflow import MlflowClient
+from mlflow.utils.arguments_utils import _get_arg_names
 from mlflow.utils.file_utils import TempDir
 from mlflow.utils.mlflow_tags import MLFLOW_PARENT_RUN_ID
-from mlflow.utils.arguments_utils import _get_arg_names
-from mlflow.utils.time_utils import get_current_time_millis
+from mlflow.utils.time import get_current_time_millis
 
 _logger = logging.getLogger(__name__)
 
@@ -45,9 +50,10 @@ def _gen_xgboost_sklearn_estimators_to_patch():
 
 
 def _gen_lightgbm_sklearn_estimators_to_patch():
-    import mlflow.lightgbm
     import lightgbm as lgb
 
+    import mlflow.lightgbm
+
     all_classes = inspect.getmembers(lgb.sklearn, inspect.isclass)
     base_class = lgb.sklearn._LGBMModelBase
     sklearn_estimators = []
@@ -65,8 +71,8 @@ def _gen_lightgbm_sklearn_estimators_to_patch():
 
 def _get_estimator_info_tags(estimator):
     """
-    :return: A dictionary of MLflow run tag keys and values
-             describing the specified estimator.
+    Returns:
+        A dictionary of MLflow run tag keys and values describing the specified estimator.
     """
     return {
         "estimator_name": estimator.__class__.__name__,
@@ -83,14 +89,16 @@ def _get_X_y_and_sample_weight(fit_func, fit_args, fit_kwargs):
        extract it from fit_args or fit_kwargs and return (X, y, sample_weight),
        otherwise return (X, y)
 
-    :param fit_func: A fit function object.
-    :param fit_args: Positional arguments given to fit_func.
-    :param fit_kwargs: Keyword arguments given to fit_func.
+    Args:
+        fit_func: A fit function object.
+        fit_args: Positional arguments given to fit_func.
+        fit_kwargs: Keyword arguments given to fit_func.
 
-    :returns: A tuple of either (X, y, sample_weight), where `y` and `sample_weight` may be
-              `None` if the specified `fit_args` and `fit_kwargs` do not specify labels or
-              a sample weighting. Copies of `X` and `y` are made in order to avoid mutation
-              of the dataset during training.
+    Returns:
+        A tuple of either (X, y, sample_weight), where `y` and `sample_weight` may be
+        `None` if the specified `fit_args` and `fit_kwargs` do not specify labels or
+        a sample weighting. Copies of `X` and `y` are made in order to avoid mutation
+        of the dataset during training.
     """
 
     def _get_Xy(args, kwargs, X_var_name, y_var_name):
@@ -149,7 +157,7 @@ def _get_metrics_value_dict(metrics_list):
     return metric_value_dict
 
 
-def _get_classifier_metrics(fitted_estimator, prefix, X, y_true, sample_weight, pos_label):
+def _get_classifier_metrics(fitted_estimator, prefix, X, y_true, sample_weight, pos_label):  # noqa: D417
     """
     Compute and record various common metrics for classifiers
 
@@ -183,10 +191,13 @@ def _get_classifier_metrics(fitted_estimator, prefix, X, y_true, sample_weight,
     (y_true, y_pred, ...... sample_weight), otherwise as (y_true, y_pred, ......)
     3. return a dictionary of metric(name, value)
 
-    :param fitted_estimator: The already fitted classifier
-    :param fit_args: Positional arguments given to fit_func.
-    :param fit_kwargs: Keyword arguments given to fit_func.
-    :return: dictionary of (function name, computed value)
+    Args:
+        fitted_estimator: The already fitted classifier
+        fit_args: Positional arguments given to fit_func.
+        fit_kwargs: Keyword arguments given to fit_func.
+
+    Returns:
+        dictionary of (function name, computed value)
     """
     import sklearn
 
@@ -287,7 +298,7 @@ def _get_class_labels_from_estimator(estimator):
     return estimator.classes_ if hasattr(estimator, "classes_") else None
 
 
-def _get_classifier_artifacts(fitted_estimator, prefix, X, y_true, sample_weight):
+def _get_classifier_artifacts(fitted_estimator, prefix, X, y_true, sample_weight):  # noqa: D417
     """
     Draw and record various common artifacts for classifier
 
@@ -308,10 +319,13 @@ def _get_classifier_artifacts(fitted_estimator, prefix, X, y_true, sample_weight
     (y_true, y_pred, sample_weight, multioutput), otherwise as (y_true, y_pred, multioutput)
     3. return a list of artifacts path to be logged
 
-    :param fitted_estimator: The already fitted regressor
-    :param fit_args: Positional arguments given to fit_func.
-    :param fit_kwargs: Keyword arguments given to fit_func.
-    :return: List of artifacts to be logged
+    Args:
+        fitted_estimator: The already fitted regressor
+        fit_args: Positional arguments given to fit_func.
+        fit_kwargs: Keyword arguments given to fit_func.
+
+    Returns:
+        List of artifacts to be logged
     """
     import sklearn
 
@@ -396,7 +410,7 @@ def _get_classifier_artifacts(fitted_estimator, prefix, X, y_true, sample_weight
     return classifier_artifacts
 
 
-def _get_regressor_metrics(fitted_estimator, prefix, X, y_true, sample_weight):
+def _get_regressor_metrics(fitted_estimator, prefix, X, y_true, sample_weight):  # noqa: D417
     """
     Compute and record various common metrics for regressors
 
@@ -416,10 +430,13 @@ def _get_regressor_metrics(fitted_estimator, prefix, X, y_true, sample_weight):
     (y_true, y_pred, sample_weight, multioutput), otherwise as (y_true, y_pred, multioutput)
     3. return a dictionary of metric(name, value)
 
-    :param fitted_estimator: The already fitted regressor
-    :param fit_args: Positional arguments given to fit_func.
-    :param fit_kwargs: Keyword arguments given to fit_func.
-    :return: dictionary of (function name, computed value)
+    Args:
+        fitted_estimator: The already fitted regressor
+        fit_args: Positional arguments given to fit_func.
+        fit_kwargs: Keyword arguments given to fit_func.
+
+    Returns:
+        dictionary of (function name, computed value)
     """
     import sklearn
 
@@ -539,7 +556,7 @@ def _log_specialized_estimator_content(
             import matplotlib.pyplot as plt
         except ImportError as ie:
             _logger.warning(
-                f"Failed to import matplotlib (error: {repr(ie)}). Skipping artifact logging."
+                f"Failed to import matplotlib (error: {ie!r}). Skipping artifact logging."
             )
             return metrics
 
@@ -550,7 +567,7 @@ def _log_specialized_estimator_content(
                     with matplotlib.rc_context(_matplotlib_config):
                         display = artifact.function(**artifact.arguments)
                         display.ax_.set_title(artifact.title)
-                        artifact_path = "{}.png".format(artifact.name)
+                        artifact_path = f"{artifact.name}.png"
                         filepath = tmp_dir.path(artifact_path)
                         display.figure_.savefig(fname=filepath, format="png")
                         plt.close(display.figure_)
@@ -605,20 +622,23 @@ def _log_estimator_content(
     tailored to the estimator's type (e.g., regression vs classification). Training labels
     are required for metric computation; metrics will be omitted if labels are not available.
 
-    :param autologging_client: An instance of `MlflowAutologgingQueueingClient` used for
-                               efficiently logging run data to MLflow Tracking.
-    :param estimator: The estimator used to compute metrics and artifacts.
-    :param run_id: The run under which the content is logged.
-    :param prefix: A prefix used to name the logged content. Typically it's 'training_' for
-                   training-time content and user-controlled for evaluation-time content.
-    :param X: The data samples.
-    :param y_true: Labels.
-    :param sample_weight: Per-sample weights used in the computation of metrics and artifacts.
-    :param pos_label: The positive label used to compute binary classification metrics such as
-        precision, recall, f1, etc. This parameter is only used for classification metrics.
-        If set to `None`, the function will calculate metrics for each label and find their
-        average weighted by support (number of true instances for each label).
-    :return: A dict of the computed metrics.
+    Args:
+        autologging_client: An instance of `MlflowAutologgingQueueingClient` used for
+            efficiently logging run data to MLflow Tracking.
+        estimator: The estimator used to compute metrics and artifacts.
+        run_id: The run under which the content is logged.
+        prefix: A prefix used to name the logged content. Typically it's 'training_' for
+            training-time content and user-controlled for evaluation-time content.
+        X: The data samples.
+        y_true: Labels.
+        sample_weight: Per-sample weights used in the computation of metrics and artifacts.
+        pos_label: The positive label used to compute binary classification metrics such as
+            precision, recall, f1, etc. This parameter is only used for classification metrics.
+            If set to `None`, the function will calculate metrics for each label and find their
+            average weighted by support (number of true instances for each label).
+
+    Returns:
+        A dict of the computed metrics.
     """
     metrics = _log_specialized_estimator_content(
         autologging_client=autologging_client,
@@ -656,9 +676,10 @@ def _log_estimator_content(
 
 def _get_meta_estimators_for_autologging():
     """
-    :return: A list of meta estimator class definitions
-             (e.g., `sklearn.model_selection.GridSearchCV`) that should be included
-             when patching training functions for autologging
+    Returns:
+        A list of meta estimator class definitions
+        (e.g., `sklearn.model_selection.GridSearchCV`) that should be included
+        when patching training functions for autologging
     """
     from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
     from sklearn.pipeline import Pipeline
@@ -672,8 +693,9 @@ def _get_meta_estimators_for_autologging():
 
 def _is_parameter_search_estimator(estimator):
     """
-    :return: `True` if the specified scikit-learn estimator is a parameter search estimator,
-             such as `GridSearchCV`. `False` otherwise.
+    Returns:
+        `True` if the specified scikit-learn estimator is a parameter search estimator,
+        such as `GridSearchCV`. `False` otherwise.
     """
     from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
 
@@ -693,11 +715,12 @@ def _log_parameter_search_results_as_artifact(cv_results_df, run_id):
     Records a collection of parameter search results as an MLflow artifact
     for the specified run.
 
-    :param cv_results_df: A Pandas DataFrame containing the results of a parameter search
-                          training session, which may be obtained by parsing the `cv_results_`
-                          attribute of a trained parameter search estimator such as
-                          `GridSearchCV`.
-    :param run_id: The ID of the MLflow Run to which the artifact should be recorded.
+    Args:
+        cv_results_df: A Pandas DataFrame containing the results of a parameter search
+            training session, which may be obtained by parsing the `cv_results_`
+            attribute of a trained parameter search estimator such as
+            `GridSearchCV`.
+        run_id: The ID of the MLflow Run to which the artifact should be recorded.
     """
     with TempDir() as t:
         results_path = t.path("cv_results.csv")
@@ -728,7 +751,7 @@ def _log_child_runs_info(max_tuning_runs, total_runs):
     _logger.info("Logging %s, %s will be omitted.", logging_phrase, omitting_phrase)
 
 
-def _create_child_runs_for_parameter_search(
+def _create_child_runs_for_parameter_search(  # noqa: D417
     autologging_client, cv_estimator, parent_run, max_tuning_runs, child_tags=None
 ):
     """
@@ -739,14 +762,15 @@ def _create_child_runs_for_parameter_search(
     for each point in the parameter search space. For additional information, see
     `https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html`_.
 
-    :param autologging_client: An instance of `MlflowAutologgingQueueingClient` used for
-                               efficiently logging run data to MLflow Tracking.
-    :param cv_estimator: The trained parameter search estimator for which to create
-                         child runs.
-    :param parent_run: A py:class:`mlflow.entities.Run` object referring to the parent
-                       parameter search run for which child runs should be created.
-    :param child_tags: An optional dictionary of MLflow tag keys and values to log
-                       for each child run.
+    Args:
+        autologging_client: An instance of `MlflowAutologgingQueueingClient` used for
+            efficiently logging run data to MLflow Tracking.
+        cv_estimator: The trained parameter search estimator for which to create
+            child runs.
+        parent_run: A py:class:`mlflow.entities.Run` object referring to the parent
+            parameter search run for which child runs should be created.
+        child_tags: An optional dictionary of MLflow tag keys and values to log
+            for each child run.
     """
     import pandas as pd
 
@@ -879,6 +903,7 @@ def _backported_all_estimators(type_filter=None):
         'classifier', 'regressor', 'cluster' and 'transformer' to get
         estimators only of these specific types, or a list of these to
         get the estimators that fit at least one of the types.
+
     Returns
     -------
     estimators : list of tuples
@@ -886,21 +911,15 @@ def _backported_all_estimators(type_filter=None):
         and ``class`` is the actual type of the class.
     """
     # lazy import to avoid circular imports from sklearn.base
-    import pkgutil
-    import platform
     import sklearn
-    from importlib import import_module
-    from operator import itemgetter
-
-    # pylint: disable=no-name-in-module, import-error
-    from sklearn.utils.testing import ignore_warnings
     from sklearn.base import (
         BaseEstimator,
         ClassifierMixin,
+        ClusterMixin,
         RegressorMixin,
         TransformerMixin,
-        ClusterMixin,
     )
+    from sklearn.utils._testing import ignore_warnings
 
     IS_PYPY = platform.python_implementation() == "PyPy"
 
@@ -941,10 +960,8 @@ def _backported_all_estimators(type_filter=None):
     estimators = [c for c in estimators if not is_abstract(c[1])]
 
     if type_filter is not None:
-        if not isinstance(type_filter, list):
-            type_filter = [type_filter]
-        else:
-            type_filter = list(type_filter)  # copy
+        # copy the object if type_filter is a list
+        type_filter = list(type_filter) if isinstance(type_filter, list) else [type_filter]
         filtered_estimators = []
         filters = {
             "classifier": ClassifierMixin,
@@ -962,7 +979,7 @@ def _backported_all_estimators(type_filter=None):
                 "Parameter type_filter must be 'classifier', "
                 "'regressor', 'transformer', 'cluster' or "
                 "None, got"
-                " %s." % repr(type_filter)
+                f" {type_filter!r}"
             )
 
     # drop duplicates, sort for reproducibility
