@@ -1,16 +1,15 @@
-import os
 import ftplib
-from ftplib import FTP
-from contextlib import contextmanager
-
+import os
 import posixpath
 import urllib.parse
+from contextlib import contextmanager
+from ftplib import FTP
+from urllib.parse import unquote
 
 from mlflow.entities.file_info import FileInfo
+from mlflow.exceptions import MlflowException
 from mlflow.store.artifact.artifact_repo import ArtifactRepository
 from mlflow.utils.file_utils import relative_path_to_artifact_path
-from mlflow.exceptions import MlflowException
-from urllib.parse import unquote
 
 
 class FTPArtifactRepository(ArtifactRepository):
@@ -25,7 +24,7 @@ class FTPArtifactRepository(ArtifactRepository):
             "username": parsed.username,
             "password": parsed.password,
         }
-        self.path = parsed.path
+        self.path = parsed.path or "/"
 
         if self.config["host"] is None:
             self.config["host"] = "localhost"
@@ -107,10 +106,10 @@ class FTPArtifactRepository(ArtifactRepository):
             if not self._is_dir(ftp, list_dir):
                 return []
             artifact_files = ftp.nlst(list_dir)
-            artifact_files = list(filter(lambda x: x != "." and x != "..", artifact_files))
             # Make sure artifact_files is a list of file names because ftp.nlst
             # may return absolute paths.
             artifact_files = [os.path.basename(f) for f in artifact_files]
+            artifact_files = list(filter(lambda x: x != "." and x != "..", artifact_files))
             infos = []
             for file_name in artifact_files:
                 file_path = file_name if path is None else posixpath.join(path, file_name)
