@@ -1,15 +1,21 @@
-from distutils import dir_util
 import os
 import shutil
+from typing import Any
 
-from mlflow.store.artifact.artifact_repo import ArtifactRepository, verify_artifact_path
+from mlflow.store.artifact.artifact_repo import (
+    ArtifactRepository,
+    try_read_trace_data,
+    verify_artifact_path,
+)
+from mlflow.tracing.artifact_utils import TRACE_DATA_FILE_NAME
 from mlflow.utils.file_utils import (
-    mkdir,
-    list_all,
     get_file_info,
+    list_all,
     local_file_uri_to_path,
+    mkdir,
     relative_path_to_artifact_path,
 )
+from mlflow.utils.uri import validate_path_is_safe
 
 
 class LocalArtifactRepository(ArtifactRepository):
@@ -35,7 +41,10 @@ class LocalArtifactRepository(ArtifactRepository):
         )
         if not os.path.exists(artifact_dir):
             mkdir(artifact_dir)
-        shutil.copyfile(local_file, os.path.join(artifact_dir, os.path.basename(local_file)))
+        try:
+            shutil.copy2(local_file, os.path.join(artifact_dir, os.path.basename(local_file)))
+        except shutil.SameFileError:
+            pass
 
     def _is_directory(self, artifact_path):
         # NOTE: The path is expected to be in posix format.
@@ -55,7 +64,7 @@ class LocalArtifactRepository(ArtifactRepository):
         )
         if not os.path.exists(artifact_dir):
             mkdir(artifact_dir)
-        dir_util.copy_tree(src=local_dir, dst=artifact_dir, preserve_mode=0, preserve_times=0)
+        shutil.copytree(src=local_dir, dst=artifact_dir, dirs_exist_ok=True)
 
     def download_artifacts(self, artifact_path, dst_path=None):
         """
@@ -63,17 +72,20 @@ class LocalArtifactRepository(ArtifactRepository):
         If ``dst_path`` is ``None``, the absolute filesystem path of the specified artifact is
         returned. If ``dst_path`` is not ``None``, the local artifact is copied to ``dst_path``.
 
-        :param artifact_path: Relative source path to the desired artifacts.
-        :param dst_path: Absolute path of the local filesystem destination directory to which to
-                         download the specified artifacts. This directory must already exist. If
-                         unspecified, the absolute path of the local artifact will be returned.
+        Args:
+            artifact_path: Relative source path to the desired artifacts.
+            dst_path: Absolute path of the local filesystem destination directory to which to
+                download the specified artifacts. This directory must already exist. If
+                unspecified, the absolute path of the local artifact will be returned.
 
-        :return: Absolute path of the local filesystem location containing the desired artifacts.
+        Returns:
+            Absolute path of the local filesystem location containing the desired artifacts.
         """
         if dst_path:
             return super().download_artifacts(artifact_path, dst_path)
-        # NOTE: The artifact_path is expected to be in posix format.
+        # NOTE: The artifact_path is expected to be a relative path in posix format.
         # Posix paths work fine on windows but just in case we normalize it here.
+        artifact_path = validate_path_is_safe(artifact_path)
         local_artifact_path = os.path.join(self.artifact_dir, os.path.normpath(artifact_path))
         if not os.path.exists(local_artifact_path):
             raise OSError(f"No such file or directory: '{local_artifact_path}'")
@@ -98,10 +110,11 @@ class LocalArtifactRepository(ArtifactRepository):
             return []
 
     def _download_file(self, remote_file_path, local_path):
-        # NOTE: The remote_file_path is expected to be in posix format.
+        # NOTE: The remote_file_path is expected to be a relative path in posix format.
         # Posix paths work fine on windows but just in case we normalize it here.
+        remote_file_path = validate_path_is_safe(remote_file_path)
         remote_file_path = os.path.join(self.artifact_dir, os.path.normpath(remote_file_path))
-        shutil.copyfile(remote_file_path, local_path)
+        shutil.copy2(remote_file_path, local_path)
 
     def delete_artifacts(self, artifact_path=None):
         artifact_path = local_file_uri_to_path(
@@ -109,4 +122,21 @@ class LocalArtifactRepository(ArtifactRepository):
         )
 
         if os.path.exists(artifact_path):
-            shutil.rmtree(artifact_path)
+            if os.path.isfile(artifact_path):
+                os.remove(artifact_path)
+            else:
+                shutil.rmtree(artifact_path)
+
+    def download_trace_data(self) -> dict[str, Any]:
+        """
+        Download the trace data.
+
+        Returns:
+            The trace data as a dictionary.
+
+        Raises:
+            - `MlflowTraceDataNotFound`: The trace data is not found.
+            - `MlflowTraceDataCorrupted`: The trace data is corrupted.
+        """
+        trace_data_path = os.path.join(self.artifact_dir, TRACE_DATA_FILE_NAME)
+        return try_read_trace_data(trace_data_path)
