@@ -1,37 +1,22 @@
-import os
-from functools import partial
 import logging
-from pathlib import Path
-from typing import Union
+import os
+from collections import OrderedDict
 from contextlib import contextmanager
+from functools import partial
+from pathlib import Path
+from typing import Generator, Union
 
-from mlflow.environment_variables import MLFLOW_TRACKING_AWS_SIGV4
-from mlflow.store.tracking import DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH
+from mlflow.environment_variables import MLFLOW_TRACKING_URI
 from mlflow.store.db.db_types import DATABASE_ENGINES
+from mlflow.store.tracking import DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH
 from mlflow.store.tracking.file_store import FileStore
 from mlflow.store.tracking.rest_store import RestStore
+from mlflow.tracing.provider import reset
 from mlflow.tracking._tracking_service.registry import TrackingStoreRegistry
-from mlflow.utils import env, rest_utils
-from mlflow.utils.file_utils import path_to_local_file_uri
+from mlflow.utils.credentials import get_default_host_creds
 from mlflow.utils.databricks_utils import get_databricks_host_creds
-from mlflow.utils.uri import _DATABRICKS_UNITY_CATALOG_SCHEME
-
-_TRACKING_URI_ENV_VAR = "MLFLOW_TRACKING_URI"
-
-# Extra environment variables which take precedence for setting the basic/bearer
-# auth on http requests.
-_TRACKING_USERNAME_ENV_VAR = "MLFLOW_TRACKING_USERNAME"
-_TRACKING_PASSWORD_ENV_VAR = "MLFLOW_TRACKING_PASSWORD"
-_TRACKING_TOKEN_ENV_VAR = "MLFLOW_TRACKING_TOKEN"
-
-# sets verify param of 'requests.request' function
-# see https://requests.readthedocs.io/en/master/api/
-_TRACKING_INSECURE_TLS_ENV_VAR = "MLFLOW_TRACKING_INSECURE_TLS"
-_TRACKING_SERVER_CERT_PATH_ENV_VAR = "MLFLOW_TRACKING_SERVER_CERT_PATH"
-
-# sets cert param of 'requests.request' function
-# see https://requests.readthedocs.io/en/master/api/
-_TRACKING_CLIENT_CERT_PATH_ENV_VAR = "MLFLOW_TRACKING_CLIENT_CERT_PATH"
+from mlflow.utils.file_utils import path_to_local_file_uri
+from mlflow.utils.uri import _DATABRICKS_UNITY_CATALOG_SCHEME, _OSS_UNITY_CATALOG_SCHEME
 
 _logger = logging.getLogger(__name__)
 _tracking_uri = None
@@ -39,7 +24,7 @@ _tracking_uri = None
 
 def is_tracking_uri_set():
     """Returns True if the tracking URI has been set, False otherwise."""
-    if _tracking_uri or env.get_env(_TRACKING_URI_ENV_VAR):
+    if _tracking_uri or MLFLOW_TRACKING_URI.get():
         return True
     return False
 
@@ -49,25 +34,26 @@ def set_tracking_uri(uri: Union[str, Path]) -> None:
     Set the tracking server URI. This does not affect the
     currently active run (if one exists), but takes effect for successive runs.
 
-    :param uri:
+    Args:
+        uri:
 
-                - An empty string, or a local file path, prefixed with ``file:/``. Data is stored
-                  locally at the provided file (or ``./mlruns`` if empty).
-                - An HTTP URI like ``https://my-tracking-server:5000``.
-                - A Databricks workspace, provided as the string "databricks" or, to use a
-                  Databricks CLI
-                  `profile <https://github.com/databricks/databricks-cli#installation>`_,
-                  "databricks://<profileName>".
-                - A :py:class:`pathlib.Path` instance
+            - An empty string, or a local file path, prefixed with ``file:/``. Data is stored
+              locally at the provided file (or ``./mlruns`` if empty).
+            - An HTTP URI like ``https://my-tracking-server:5000``.
+            - A Databricks workspace, provided as the string "databricks" or, to use a Databricks
+              CLI `profile <https://github.com/databricks/databricks-cli#installation>`_,
+              "databricks://<profileName>".
+            - A :py:class:`pathlib.Path` instance
 
-    .. test-code-block:: python
+    .. code-block:: python
+        :test:
         :caption: Example
 
         import mlflow
 
         mlflow.set_tracking_uri("file:///tmp/my_tracking")
         tracking_uri = mlflow.get_tracking_uri()
-        print("Current tracking uri: {}".format(tracking_uri))
+        print(f"Current tracking uri: {tracking_uri}")
 
     .. code-block:: text
         :caption: Output
@@ -81,27 +67,36 @@ def set_tracking_uri(uri: Union[str, Path]) -> None:
         # then .resolve() to clean the path
         uri = uri.absolute().resolve().as_uri()
     global _tracking_uri
-    _tracking_uri = uri
+
+    if _tracking_uri != uri:
+        _tracking_uri = uri
+        if _tracking_uri is not None:
+            # Set 'MLFLOW_TRACKING_URI' environment variable
+            # so that subprocess can inherit it.
+            MLFLOW_TRACKING_URI.set(_tracking_uri)
+        else:
+            MLFLOW_TRACKING_URI.unset()
+
+        # Tracer provider uses tracking URI to determine where to export traces.
+        # Tracer provider stores the URI as its state so we need to reset
+        # it explicitly when the global tracking URI changes.
+        reset()
 
 
 @contextmanager
-def _use_tracking_uri(uri: str, local_store_root_path: str = None) -> None:
+def _use_tracking_uri(uri: str) -> Generator[None, None, None]:
+    """Temporarily use the specified tracking URI.
+
+    Args:
+        uri: The tracking URI to use.
+
     """
-    Similar to `mlflow.tracking.set_tracking_uri` function but return a context manager.
-    :param uri: tracking URI to use.
-    :param local_store_root_path: the local store root path for the tracking URI.
-    """
-    global _tracking_uri
-    cwd = os.getcwd()
     old_tracking_uri = _tracking_uri
     try:
-        if local_store_root_path is not None:
-            os.chdir(local_store_root_path)
-        _tracking_uri = uri
+        set_tracking_uri(uri)
         yield
     finally:
-        _tracking_uri = old_tracking_uri
-        os.chdir(cwd)
+        set_tracking_uri(old_tracking_uri)
 
 
 def _resolve_tracking_uri(tracking_uri=None):
@@ -109,31 +104,28 @@ def _resolve_tracking_uri(tracking_uri=None):
 
 
 def get_tracking_uri() -> str:
-    """
-    Get the current tracking URI. This may not correspond to the tracking URI of
+    """Get the current tracking URI. This may not correspond to the tracking URI of
     the currently active run, since the tracking URI can be updated via ``set_tracking_uri``.
 
-    :return: The tracking URI.
+    Returns:
+        The tracking URI.
 
     .. code-block:: python
-        :caption: Example
 
         import mlflow
 
         # Get the current tracking uri
         tracking_uri = mlflow.get_tracking_uri()
-        print("Current tracking uri: {}".format(tracking_uri))
+        print(f"Current tracking uri: {tracking_uri}")
 
     .. code-block:: text
-        :caption: Output
 
         Current tracking uri: file:///.../mlruns
     """
-    global _tracking_uri
     if _tracking_uri is not None:
         return _tracking_uri
-    elif env.get_env(_TRACKING_URI_ENV_VAR) is not None:
-        return env.get_env(_TRACKING_URI_ENV_VAR)
+    elif uri := MLFLOW_TRACKING_URI.get():
+        return uri
     else:
         return path_to_local_file_uri(os.path.abspath(DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH))
 
@@ -150,21 +142,8 @@ def _get_sqlalchemy_store(store_uri, artifact_uri):
     return SqlAlchemyStore(store_uri, artifact_uri)
 
 
-def _get_default_host_creds(store_uri):
-    return rest_utils.MlflowHostCreds(
-        host=store_uri,
-        username=os.environ.get(_TRACKING_USERNAME_ENV_VAR),
-        password=os.environ.get(_TRACKING_PASSWORD_ENV_VAR),
-        token=os.environ.get(_TRACKING_TOKEN_ENV_VAR),
-        aws_sigv4=MLFLOW_TRACKING_AWS_SIGV4.get(),
-        ignore_tls_verification=os.environ.get(_TRACKING_INSECURE_TLS_ENV_VAR) == "true",
-        client_cert_path=os.environ.get(_TRACKING_CLIENT_CERT_PATH_ENV_VAR),
-        server_cert_path=os.environ.get(_TRACKING_SERVER_CERT_PATH_ENV_VAR),
-    )
-
-
 def _get_rest_store(store_uri, **_):
-    return RestStore(partial(_get_default_host_creds, store_uri))
+    return RestStore(partial(get_default_host_creds, store_uri))
 
 
 def _get_databricks_rest_store(store_uri, **_):
@@ -175,11 +154,10 @@ def _get_databricks_uc_rest_store(store_uri, **_):
     from mlflow.exceptions import MlflowException
     from mlflow.version import VERSION
 
-    global _tracking_store_registry
     supported_schemes = [
         scheme
         for scheme in _tracking_store_registry._registry
-        if scheme != _DATABRICKS_UNITY_CATALOG_SCHEME
+        if scheme not in {_DATABRICKS_UNITY_CATALOG_SCHEME, _OSS_UNITY_CATALOG_SCHEME}
     ]
     raise MlflowException(
         f"Detected Unity Catalog tracking URI '{store_uri}'. "
@@ -191,46 +169,71 @@ def _get_databricks_uc_rest_store(store_uri, **_):
         "Catalog, please upgrade to the latest version of the MLflow Python "
         "client, then specify a Unity Catalog model registry URI via "
         f"mlflow.set_registry_uri('{_DATABRICKS_UNITY_CATALOG_SCHEME}') or "
-        f"mlflow.set_registry_uri('{_DATABRICKS_UNITY_CATALOG_SCHEME}://profile_name'), where "
+        f"mlflow.set_registry_uri('{_DATABRICKS_UNITY_CATALOG_SCHEME}://profile_name') where "
         "'profile_name' is the name of the Databricks CLI profile to use for "
-        "authentication. Be sure to leave the tracking URI configured to use "
-        "one of the supported schemes listed above."
+        "authentication. A OSS Unity Catalog model registry URI can also be specified via "
+        f"mlflow.set_registry_uri('{_OSS_UNITY_CATALOG_SCHEME}:http://localhost:8080')."
+        "Be sure to leave the registry URI configured to use one of the supported"
+        "schemes listed above."
     )
 
 
 _tracking_store_registry = TrackingStoreRegistry()
-_tracking_store_registry.register("", _get_file_store)
-_tracking_store_registry.register("file", _get_file_store)
-_tracking_store_registry.register("databricks", _get_databricks_rest_store)
-_tracking_store_registry.register(_DATABRICKS_UNITY_CATALOG_SCHEME, _get_databricks_uc_rest_store)
 
-for scheme in ["http", "https"]:
-    _tracking_store_registry.register(scheme, _get_rest_store)
 
-for scheme in DATABASE_ENGINES:
-    _tracking_store_registry.register(scheme, _get_sqlalchemy_store)
+def _register_tracking_stores():
+    _tracking_store_registry.register("", _get_file_store)
+    _tracking_store_registry.register("file", _get_file_store)
+    _tracking_store_registry.register("databricks", _get_databricks_rest_store)
+    _tracking_store_registry.register(
+        _DATABRICKS_UNITY_CATALOG_SCHEME, _get_databricks_uc_rest_store
+    )
+    _tracking_store_registry.register(_OSS_UNITY_CATALOG_SCHEME, _get_databricks_uc_rest_store)
 
-_tracking_store_registry.register_entrypoints()
+    for scheme in ["http", "https"]:
+        _tracking_store_registry.register(scheme, _get_rest_store)
+
+    for scheme in DATABASE_ENGINES:
+        _tracking_store_registry.register(scheme, _get_sqlalchemy_store)
+
+    _tracking_store_registry.register_entrypoints()
+
+
+def _register(scheme, builder):
+    _tracking_store_registry.register(scheme, builder)
+
+
+_register_tracking_stores()
 
 
 def _get_store(store_uri=None, artifact_uri=None):
     return _tracking_store_registry.get_store(store_uri, artifact_uri)
 
 
+_artifact_repos_cache = OrderedDict()
+
+
+def _get_artifact_repo(run_id):
+    return _artifact_repos_cache.get(run_id)
+
+
 # TODO(sueann): move to a projects utils module
 def _get_git_url_if_present(uri):
-    """
-    Return the path git_uri#sub_directory if the URI passed is a local path that's part of
+    """Return the path git_uri#sub_directory if the URI passed is a local path that's part of
     a Git repo, or returns the original URI otherwise.
-    :param uri: The expanded uri
-    :return: The git_uri#sub_directory if the uri is part of a Git repo,
-             otherwise return the original uri
+
+    Args:
+        uri: The expanded uri.
+
+    Returns:
+        The git_uri#sub_directory if the uri is part of a Git repo, otherwise return the original
+        uri.
     """
     if "#" in uri:
         # Already a URI in git repo format
         return uri
     try:
-        from git import Repo, InvalidGitRepositoryError, GitCommandNotFound, NoSuchPathError
+        from git import GitCommandNotFound, InvalidGitRepositoryError, NoSuchPathError, Repo
     except ImportError as e:
         _logger.warning(
             "Failed to import Git (the git executable is probably not on your PATH),"
@@ -243,7 +246,7 @@ def _get_git_url_if_present(uri):
         repo = Repo(uri, search_parent_directories=True)
 
         # Repo url
-        repo_url = "file://%s" % repo.working_tree_dir
+        repo_url = f"file://{repo.working_tree_dir}"
 
         # Sub directory
         rlpath = uri.replace(repo.working_tree_dir, "")
