@@ -1,15 +1,14 @@
+import inspect
 import logging
+import socket
 import subprocess
+from contextlib import closing
 from itertools import islice
 from sys import version_info
 
+from mlflow.utils.pydantic_utils import IS_PYDANTIC_V2_OR_NEWER  # noqa: F401
 
-_logger = logging.getLogger(__name__)
-
-
-PYTHON_VERSION = "{major}.{minor}.{micro}".format(
-    major=version_info.major, minor=version_info.minor, micro=version_info.micro
-)
+PYTHON_VERSION = f"{version_info.major}.{version_info.minor}.{version_info.micro}"
 
 
 _logger = logging.getLogger(__name__)
@@ -19,37 +18,6 @@ def get_major_minor_py_version(py_version):
     return ".".join(py_version.split(".")[:2])
 
 
-def get_unique_resource_id(max_length=None):
-    """
-    Obtains a unique id that can be included in a resource name. This unique id is a valid
-    DNS subname.
-
-    :param max_length: The maximum length of the identifier
-    :return: A unique identifier that can be appended to a user-readable resource name to avoid
-             naming collisions.
-    """
-    import uuid
-    import base64
-
-    if max_length is not None and max_length <= 0:
-        raise ValueError(
-            "The specified maximum length for the unique resource id must be positive!"
-        )
-
-    uuid_bytes = uuid.uuid4().bytes
-    # Use base64 encoding to shorten the UUID length. Note that the replacement of the
-    # unsupported '+' symbol maintains uniqueness because the UUID byte string is of a fixed,
-    # 16-byte length
-    uuid_b64 = base64.b64encode(uuid_bytes)
-    # In Python3, `uuid_b64` is a `bytes` object. It needs to be
-    # converted to a string
-    uuid_b64 = uuid_b64.decode("ascii")
-    unique_id = uuid_b64.rstrip("=\n").replace("/", "-").replace("+", "AB").lower()
-    if max_length is not None:
-        unique_id = unique_id[: int(max_length)]
-    return unique_id
-
-
 def reraise(tp, value, tb=None):
     # Taken from: https://github.com/benjaminp/six/blob/1.15.0/six.py#L694-L700
     try:
@@ -124,15 +92,17 @@ def _truncate_dict(d, max_key_length=None, max_value_length=None):
 
 
 def merge_dicts(dict_a, dict_b, raise_on_duplicates=True):
-    """
-    This function takes two dictionaries and returns one singular merged dictionary.
+    """This function takes two dictionaries and returns one singular merged dictionary.
+
+    Args:
+        dict_a: The first dictionary.
+        dict_b: The second dictionary.
+        raise_on_duplicates: If True, the function raises ValueError if there are duplicate keys.
+            Otherwise, duplicate keys in `dict_b` will override the ones in `dict_a`.
+
+    Returns:
+        A merged dictionary.
 
-    :param dict_a: The first dictionary.
-    :param dict_b: The second dictonary.
-    :param raise_on_duplicates: If True, the function raises ValueError if there are duplicate keys.
-                                Otherwise, duplicate keys in `dict_b` will override the ones in
-                                `dict_a`.
-    :return: A merged dictionary.
     """
     duplicate_keys = dict_a.keys() & dict_b.keys()
     if raise_on_duplicates and len(duplicate_keys) > 0:
@@ -153,8 +123,6 @@ def _inspect_original_var_name(var, fallback_name):
     in the most outer frame.
     If inspect failed, return fallback_name
     """
-    import inspect
-
     if var is None:
         return fallback_name
     try:
@@ -162,7 +130,7 @@ def _inspect_original_var_name(var, fallback_name):
 
         frame = inspect.currentframe().f_back
         while frame is not None:
-            arg_info = inspect.getargvalues(frame)  # pylint: disable=deprecated-method
+            arg_info = inspect.getargvalues(frame)
 
             fixed_args = [arg_info.locals[arg_name] for arg_name in arg_info.args]
             varlen_args = list(arg_info.locals[arg_info.varargs]) if arg_info.varargs else []
@@ -195,9 +163,6 @@ def find_free_port():
     """
     Find free socket port on local machine.
     """
-    import socket
-    from contextlib import closing
-
     with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
         s.bind(("", 0))
         s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -227,8 +192,11 @@ def check_port_connectivity():
 
 def is_iterator(obj):
     """
-    :param obj: any object.
-    :return: boolean representing whether or not 'obj' is an iterator.
+    Args:
+        obj: Any object.
+
+    Returns:
+        Boolean representing whether or not 'obj' is an iterator.
     """
     return (hasattr(obj, "__next__") or hasattr(obj, "next")) and hasattr(obj, "__iter__")
 
@@ -243,21 +211,20 @@ def _is_in_ipython_notebook():
 
 
 def get_results_from_paginated_fn(paginated_fn, max_results_per_page, max_results=None):
-    """
-    Gets results by calling the ``paginated_fn`` until either no more results remain or
+    """Gets results by calling the ``paginated_fn`` until either no more results remain or
     the specified ``max_results`` threshold has been reached.
 
-    :param paginated_fn:
-    :type paginated_fn: This function is expected to take in the number of results to retrieve
-        per page and a pagination token, and return a PagedList object
-    :param max_results_per_page:
-    :type max_results_per_page: The maximum number of results to retrieve per page
-    :param max_results:
-    :type max_results: The maximum number of results to retrieve overall. If unspecified,
-                       all results will be retrieved.
-    :return: Returns a list of entities, as determined by the paginated_fn parameter, with no more
-        entities than specified by max_results
-    :rtype: list[object]
+    Args:
+        paginated_fn: This function is expected to take in the number of results to retrieve
+            per page and a pagination token, and return a PagedList object.
+        max_results_per_page: The maximum number of results to retrieve per page.
+        max_results: The maximum number of results to retrieve overall. If unspecified,
+            all results will be retrieved.
+
+    Returns:
+        Returns a list of entities, as determined by the paginated_fn parameter, with no more
+        entities than specified by max_results.
+
     """
     all_results = []
     next_page_token = None
@@ -274,3 +241,31 @@ def get_results_from_paginated_fn(paginated_fn, max_results_per_page, max_result
         else:
             break
     return all_results
+
+
+class AttrDict(dict):
+    """
+    Dict-like object that exposes its keys as attributes.
+
+    Examples
+    --------
+    >>> d = AttrDict({"a": 1, "b": 2})
+    >>> d.a
+    1
+    >>> d = AttrDict({"a": 1, "b": {"c": 3, "d": 4}})
+    >>> d.b.c
+    3
+    """
+
+    def __getattr__(self, attr):
+        try:
+            value = self[attr]
+        except KeyError:
+            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{attr}'")
+        if isinstance(value, dict):
+            return AttrDict(value)
+        return value
+
+
+def get_parent_module(module):
+    return module[0 : module.rindex(".")]
