@@ -8,27 +8,28 @@ MlflowAutologgingQueueingClient to MlflowClient in order to provide broader bene
 Remove this developer API.
 """
 
-import os
 import logging
+import os
 from collections import namedtuple
 from concurrent.futures import ThreadPoolExecutor
 from itertools import zip_longest
-from typing import Any, Dict, Optional, Union
+from typing import Any, Optional, Union
 
-from mlflow.entities import Param, RunTag, Metric
+from mlflow.entities import Metric, Param, RunTag
+from mlflow.entities.dataset_input import DatasetInput
 from mlflow.exceptions import MlflowException
 from mlflow.tracking.client import MlflowClient
-from mlflow.utils import chunk_list, _truncate_dict
+from mlflow.utils import _truncate_dict, chunk_list
+from mlflow.utils.time import get_current_time_millis
 from mlflow.utils.validation import (
+    MAX_DATASETS_PER_BATCH,
     MAX_ENTITIES_PER_BATCH,
     MAX_ENTITY_KEY_LENGTH,
-    MAX_TAG_VAL_LENGTH,
+    MAX_METRICS_PER_BATCH,
     MAX_PARAM_VAL_LENGTH,
     MAX_PARAMS_TAGS_PER_BATCH,
-    MAX_METRICS_PER_BATCH,
+    MAX_TAG_VAL_LENGTH,
 )
-from mlflow.utils.time_utils import get_current_time_millis
-
 
 _logger = logging.getLogger(__name__)
 
@@ -82,7 +83,10 @@ class RunOperations:
 # of CPU cores available on the system (whichever is smaller)
 num_cpus = os.cpu_count() or 4
 num_logging_workers = min(num_cpus * 2, 8)
-_AUTOLOGGING_QUEUEING_CLIENT_THREAD_POOL = ThreadPoolExecutor(max_workers=num_logging_workers)
+_AUTOLOGGING_QUEUEING_CLIENT_THREAD_POOL = ThreadPoolExecutor(
+    max_workers=num_logging_workers,
+    thread_name_prefix="MlflowAutologgingQueueingClient",
+)
 
 
 class MlflowAutologgingQueueingClient:
@@ -112,7 +116,7 @@ class MlflowAutologgingQueueingClient:
         """
         return self
 
-    def __exit__(self, exc_type, exc, traceback):  # pylint: disable=unused-argument
+    def __exit__(self, exc_type, exc, traceback):
         """
         Enables `MlflowAutologgingQueueingClient` to be used as a context manager with
         synchronous flushing upon exit, removing the need to call `flush()` for use cases
@@ -137,7 +141,7 @@ class MlflowAutologgingQueueingClient:
         self,
         experiment_id: str,
         start_time: Optional[int] = None,
-        tags: Optional[Dict[str, Any]] = None,
+        tags: Optional[dict[str, Any]] = None,
         run_name: Optional[str] = None,
     ) -> PendingRunId:
         """
@@ -145,8 +149,9 @@ class MlflowAutologgingQueueingClient:
         instance that can be used as input to other client logging APIs (e.g. `log_metrics`,
         `log_params`, ...).
 
-        :return: A `PendingRunId` that can be passed as the `run_id` parameter to other client
-                 logging APIs, such as `log_params` and `log_metrics`.
+        Returns:
+            A `PendingRunId` that can be passed as the `run_id` parameter to other client
+            logging APIs, such as `log_params` and `log_metrics`.
         """
         tags = tags or {}
         tags = _truncate_dict(
@@ -177,7 +182,7 @@ class MlflowAutologgingQueueingClient:
             set_terminated=_PendingSetTerminated(status=status, end_time=end_time)
         )
 
-    def log_params(self, run_id: Union[str, PendingRunId], params: Dict[str, Any]) -> None:
+    def log_params(self, run_id: Union[str, PendingRunId], params: dict[str, Any]) -> None:
         """
         Enqueues a collection of Parameters to be logged to the run specified by `run_id`.
         """
@@ -187,10 +192,20 @@ class MlflowAutologgingQueueingClient:
         params_arr = [Param(key, str(value)) for key, value in params.items()]
         self._get_pending_operations(run_id).enqueue(params=params_arr)
 
+    def log_inputs(
+        self, run_id: Union[str, PendingRunId], datasets: Optional[list[DatasetInput]]
+    ) -> None:
+        """
+        Enqueues a collection of Dataset to be logged to the run specified by `run_id`.
+        """
+        if datasets is None or len(datasets) == 0:
+            return
+        self._get_pending_operations(run_id).enqueue(datasets=datasets)
+
     def log_metrics(
         self,
         run_id: Union[str, PendingRunId],
-        metrics: Dict[str, float],
+        metrics: dict[str, float],
         step: Optional[int] = None,
     ) -> None:
         """
@@ -204,7 +219,7 @@ class MlflowAutologgingQueueingClient:
         ]
         self._get_pending_operations(run_id).enqueue(metrics=metrics_arr)
 
-    def set_tags(self, run_id: Union[str, PendingRunId], tags: Dict[str, Any]) -> None:
+    def set_tags(self, run_id: Union[str, PendingRunId], tags: dict[str, Any]) -> None:
         """
         Enqueues a collection of Tags to be logged to the run specified by `run_id`.
         """
@@ -219,15 +234,18 @@ class MlflowAutologgingQueueingClient:
         Flushes all queued run operations, resulting in the creation or mutation of runs
         and run data.
 
-        :param synchronous: If `True`, run operations are performed synchronously, and a
-                            `RunOperations` result object is only returned once all operations
-                            are complete. If `False`, run operations are performed asynchronously,
-                            and an `RunOperations` object is returned that represents the ongoing
-                            run operations.
-        :return: A `RunOperations` instance representing the flushed operations. These operations
-                 are already complete if `synchronous` is `True`. If `synchronous` is `False`, these
-                 operations may still be inflight. Operation completion can be synchronously waited
-                 on via `RunOperations.await_completion()`.
+        Args:
+            synchronous: If `True`, run operations are performed synchronously, and a
+                `RunOperations` result object is only returned once all operations
+                are complete. If `False`, run operations are performed asynchronously,
+                and an `RunOperations` object is returned that represents the ongoing
+                run operations.
+
+        Returns:
+            A `RunOperations` instance representing the flushed operations. These operations
+            are already complete if `synchronous` is `True`. If `synchronous` is `False`, these
+            operations may still be inflight. Operation completion can be synchronously waited
+            on via `RunOperations.await_completion()`.
         """
         logging_futures = []
         for pending_operations in self._pending_ops_by_run_id.values():
@@ -245,8 +263,9 @@ class MlflowAutologgingQueueingClient:
 
     def _get_pending_operations(self, run_id):
         """
-        :return: A `_PendingRunOperations` containing all pending operations for the
-                 specified `run_id`.
+        Returns:
+            A `_PendingRunOperations` containing all pending operations for the
+            specified `run_id`.
         """
         if run_id not in self._pending_ops_by_run_id:
             self._pending_ops_by_run_id[run_id] = _PendingRunOperations(run_id=run_id)
@@ -332,6 +351,13 @@ class MlflowAutologgingQueueingClient:
                 self._try_operation(self._client.log_batch, run_id=run_id, metrics=metrics_batch)
             )
 
+        for datasets_batch in chunk_list(
+            pending_operations.datasets_queue, chunk_size=MAX_DATASETS_PER_BATCH
+        ):
+            operation_results.append(
+                self._try_operation(self._client.log_inputs, run_id=run_id, datasets=datasets_batch)
+            )
+
         if pending_operations.set_terminated:
             operation_results.append(
                 self._try_operation(
@@ -364,8 +390,17 @@ class _PendingRunOperations:
         self.params_queue = []
         self.tags_queue = []
         self.metrics_queue = []
+        self.datasets_queue = []
 
-    def enqueue(self, params=None, tags=None, metrics=None, create_run=None, set_terminated=None):
+    def enqueue(
+        self,
+        params=None,
+        tags=None,
+        metrics=None,
+        datasets=None,
+        create_run=None,
+        set_terminated=None,
+    ):
         """
         Enqueues a new pending logging operation for the associated MLflow Run.
         """
@@ -379,8 +414,4 @@ class _PendingRunOperations:
         self.params_queue += params or []
         self.tags_queue += tags or []
         self.metrics_queue += metrics or []
-
-
-__all__ = [
-    "MlflowAutologgingQueueingClient",
-]
+        self.datasets_queue += datasets or []
