@@ -1,19 +1,19 @@
 import abc
+import functools
 import inspect
 import itertools
-import functools
-import os
+import typing
 import uuid
 from abc import abstractmethod
 from contextlib import contextmanager
-import typing
+from typing import Optional
 
 import mlflow
 import mlflow.utils.autologging_utils
 from mlflow.entities.run_status import RunStatus
+from mlflow.environment_variables import _MLFLOW_AUTOLOGGING_TESTING
 from mlflow.tracking.client import MlflowClient
-from mlflow.utils import gorilla
-from mlflow.utils import is_iterator
+from mlflow.utils import gorilla, is_iterator
 from mlflow.utils.autologging_utils import _logger
 from mlflow.utils.autologging_utils.events import AutologgingEventLogger
 from mlflow.utils.autologging_utils.logging_and_warnings import (
@@ -22,8 +22,6 @@ from mlflow.utils.autologging_utils.logging_and_warnings import (
 )
 from mlflow.utils.mlflow_tags import MLFLOW_AUTOLOGGING
 
-_AUTOLOGGING_TEST_MODE_ENV_VAR = "MLFLOW_AUTOLOGGING_TESTING"
-
 _AUTOLOGGING_PATCHES = {}
 
 
@@ -54,8 +52,7 @@ def exception_safe_function_for_class(function):
             else:
                 _logger.warning("Encountered unexpected error during autologging: %s", e)
 
-    safe_function = update_wrapper_extended(safe_function, function)
-    return safe_function
+    return update_wrapper_extended(safe_function, function)
 
 
 def _safe_function(function, *args, **kwargs):
@@ -147,23 +144,22 @@ class PatchFunction:
         """
         Invokes the patch function code.
 
-        :param original: The original, underlying function over which the `PatchFunction`
-                         is being applied.
-        :param *args: The positional arguments passed to the original function.
-        :param **kwargs: The keyword arguments passed to the original function.
+        Args:
+            original: The original, underlying function over which the `PatchFunction`
+                is being applied.
+            *args: The positional arguments passed to the original function.
+            **kwargs: The keyword arguments passed to the original function.
         """
-        pass
 
     @abstractmethod
     def _on_exception(self, exception):
-        """
-        Called when an unhandled standard Python exception (i.e. an exception inheriting from
+        """Called when an unhandled standard Python exception (i.e. an exception inheriting from
         `Exception`) or a `KeyboardInterrupt` prematurely terminates the execution of
         `_patch_implementation`.
 
-        :param exception: The unhandled exception thrown by `_patch_implementation`.
+        Args:
+            exception: The unhandled exception thrown by `_patch_implementation`.
         """
-        pass
 
     @classmethod
     def call(cls, original, *args, **kwargs):
@@ -182,8 +178,7 @@ class PatchFunction:
 
 
 def with_managed_run(autologging_integration, patch_function, tags=None):
-    """
-    Given a `patch_function`, returns an `augmented_patch_function` that wraps the execution of
+    """Given a `patch_function`, returns an `augmented_patch_function` that wraps the execution of
     `patch_function` with an active MLflow run. The following properties apply:
 
         - An MLflow run is only created if there is no active run present when the
@@ -199,13 +194,15 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
     is responsible for terminating them by the time it terminates
     (or in the event of an exception).
 
-    :param autologging_integration: The autologging integration associated
-                                    with the `patch_function`.
-    :param patch_function: A `PatchFunction` class definition or a function object
-                           compatible with `safe_patch`.
-    :param tags: A dictionary of string tags to set on each managed run created during the
-                 execution of `patch_function`.
+    Args:
+        autologging_integration: The autologging integration associated
+            with the `patch_function`.
+        patch_function: A `PatchFunction` class definition or a function object
+            compatible with `safe_patch`.
+        tags: A dictionary of string tags to set on each managed run created during the
+            execution of `patch_function`.
     """
+    from mlflow.utils.autologging_utils import _has_active_training_session
 
     def create_managed_run():
         managed_run = mlflow.start_run(tags=tags)
@@ -226,7 +223,11 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
                 self.managed_run = None
 
             def _patch_implementation(self, original, *args, **kwargs):
-                if not mlflow.active_run():
+                # If there is an active training session but there is no active run
+                # in current thread, it means the thread is spawned by `estimator.fit`
+                # as a worker thread, we should disable autologging in
+                # these worker threads, so skip creating managed run.
+                if not mlflow.active_run() and not _has_active_training_session():
                     self.managed_run = create_managed_run()
 
                 result = super()._patch_implementation(original, *args, **kwargs)
@@ -247,7 +248,11 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
 
         def patch_with_managed_run(original, *args, **kwargs):
             managed_run = None
-            if not mlflow.active_run():
+            # If there is an active training session but there is no active run
+            # in current thread, it means the thread is spawned by `estimator.fit`
+            # as a worker thread, we should disable autologging in
+            # these worker threads, so skip creating managed run.
+            if not mlflow.active_run() and not _has_active_training_session():
                 managed_run = create_managed_run()
 
             try:
@@ -278,42 +283,69 @@ def is_testing():
         - Disables exception handling for patched function logic, ensuring that patch code
           executes without errors during testing
     """
-    return os.environ.get(_AUTOLOGGING_TEST_MODE_ENV_VAR, "false") == "true"
+    return _MLFLOW_AUTOLOGGING_TESTING.get()
+
+
+def _resolve_extra_tags(autologging_integration, extra_tags):
+    tags = {MLFLOW_AUTOLOGGING: autologging_integration}
+    if extra_tags:
+        if isinstance(extra_tags, dict):
+            if MLFLOW_AUTOLOGGING in extra_tags:
+                extra_tags.pop(MLFLOW_AUTOLOGGING)
+                _logger.warning(
+                    f"Tag `{MLFLOW_AUTOLOGGING}` is ignored as it is a reserved tag by MLflow "
+                    f"autologging."
+                )
+            tags.update(extra_tags)
+        else:
+            raise mlflow.exceptions.MlflowException.invalid_parameter_value(
+                f"Invalid `extra_tags` type: expecting dictionary, "
+                f"received `{type(extra_tags).__name__}`"
+            )
+    return tags
 
 
 def safe_patch(
-    autologging_integration, destination, function_name, patch_function, manage_run=False
+    autologging_integration,
+    destination,
+    function_name,
+    patch_function,
+    manage_run=False,
+    extra_tags=None,
 ):
-    """
-    Patches the specified `function_name` on the specified `destination` class for autologging
+    """Patches the specified `function_name` on the specified `destination` class for autologging
     purposes, preceding its implementation with an error-safe copy of the specified patch
     `patch_function` with the following error handling behavior:
         - Exceptions thrown from the underlying / original function
           (`<destination>.<function_name>`) are propagated to the caller.
         - Exceptions thrown from other parts of the patched implementation (`patch_function`)
           are caught and logged as warnings.
-    :param autologging_integration: The name of the autologging integration associated with the
-                                    patch.
-    :param destination: The Python class on which the patch is being defined.
-    :param function_name: The name of the function to patch on the specified `destination` class.
-    :param patch_function: The patched function code to apply. This is either a `PatchFunction`
-                           class definition or a function object. If it is a function object, the
-                           first argument should be reserved for an `original` method argument
-                           representing the underlying / original function. Subsequent arguments
-                           should be identical to those of the original function being patched.
-    :param manage_run: If `True`, applies the `with_managed_run` wrapper to the specified
-                       `patch_function`, which automatically creates & terminates an MLflow
-                       active run during patch code execution if necessary. If `False`,
-                       does not apply the `with_managed_run` wrapper to the specified
-                       `patch_function`.
+
+    Args:
+        autologging_integration: The name of the autologging integration associated with the
+            patch.
+        destination: The Python class on which the patch is being defined.
+        function_name: The name of the function to patch on the specified `destination` class.
+        patch_function: The patched function code to apply. This is either a `PatchFunction`
+            class definition or a function object. If it is a function object, the
+            first argument should be reserved for an `original` method argument
+            representing the underlying / original function. Subsequent arguments
+            should be identical to those of the original function being patched.
+        manage_run: If `True`, applies the `with_managed_run` wrapper to the specified
+            `patch_function`, which automatically creates & terminates an MLflow
+            active run during patch code execution if necessary. If `False`,
+            does not apply the `with_managed_run` wrapper to the specified
+            `patch_function`.
+        extra_tags: A dictionary of extra tags to set on each managed run created by autologging.
     """
-    from mlflow.utils.autologging_utils import get_autologging_config, autologging_is_disabled
+    from mlflow.utils.autologging_utils import autologging_is_disabled, get_autologging_config
 
     if manage_run:
+        tags = _resolve_extra_tags(autologging_integration, extra_tags)
         patch_function = with_managed_run(
             autologging_integration,
             patch_function,
-            tags={MLFLOW_AUTOLOGGING: autologging_integration},
+            tags=tags,
         )
 
     patch_is_class = inspect.isclass(patch_function)
@@ -330,7 +362,7 @@ def safe_patch(
         destination, function_name, bypass_descriptor_protocol=True
     )
     if original_fn != raw_original_obj:
-        raise RuntimeError(f"Unsupport patch on {str(destination)}.{function_name}")
+        raise RuntimeError(f"Unsupported patch on {destination}.{function_name}")
     elif isinstance(original_fn, property):
         is_property_method = True
 
@@ -382,25 +414,28 @@ def safe_patch(
         # `contextlib.ContextDecorator`, which creates generator expressions that cannot be pickled
         # during model serialization by ML frameworks such as scikit-learn
         is_silent_mode = get_autologging_config(autologging_integration, "silent", False)
-        with set_mlflow_events_and_warnings_behavior_globally(
-            # MLflow warnings emitted during autologging training sessions are likely not
-            # actionable and result from the autologging implementation invoking another MLflow
-            # API. Accordingly, we reroute these warnings to the MLflow event logger with level
-            # WARNING For reference, see recommended warning and event logging behaviors from
-            # https://docs.python.org/3/howto/logging.html#when-to-use-logging
-            reroute_warnings=True,
-            disable_event_logs=is_silent_mode,
-            disable_warnings=is_silent_mode,
-        ), set_non_mlflow_warnings_behavior_for_current_thread(
-            # non-MLflow Warnings emitted during the autologging preamble (before the original /
-            # underlying ML function is called) and postamble (after the original / underlying ML
-            # function is called) are likely not actionable and result from the autologging
-            # implementation invoking an API from a dependent library. Accordingly, we reroute
-            # these warnings to the MLflow event logger with level WARNING. For reference, see
-            # recommended warning and event logging behaviors from
-            # https://docs.python.org/3/howto/logging.html#when-to-use-logging
-            reroute_warnings=True,
-            disable_warnings=is_silent_mode,
+        with (
+            set_mlflow_events_and_warnings_behavior_globally(
+                # MLflow warnings emitted during autologging training sessions are likely not
+                # actionable and result from the autologging implementation invoking another MLflow
+                # API. Accordingly, we reroute these warnings to the MLflow event logger with level
+                # WARNING For reference, see recommended warning and event logging behaviors from
+                # https://docs.python.org/3/howto/logging.html#when-to-use-logging
+                reroute_warnings=True,
+                disable_event_logs=is_silent_mode,
+                disable_warnings=is_silent_mode,
+            ),
+            set_non_mlflow_warnings_behavior_for_current_thread(
+                # non-MLflow Warnings emitted during the autologging preamble (before the original /
+                # underlying ML function is called) and postamble (after the original / underlying
+                # ML function is called) are likely not actionable and result from the autologging
+                # implementation invoking an API from a dependent library. Accordingly, we reroute
+                # these warnings to the MLflow event logger with level WARNING. For reference, see
+                # recommended warning and event logging behaviors from
+                # https://docs.python.org/3/howto/logging.html#when-to-use-logging
+                reroute_warnings=True,
+                disable_warnings=is_silent_mode,
+            ),
         ):
             if is_testing():
                 preexisting_run_for_testing = mlflow.active_run()
@@ -420,7 +455,10 @@ def safe_patch(
                 active_session_failed
                 or autologging_is_disabled(autologging_integration)
                 or (user_created_fluent_run_is_active and exclusive)
-                or mlflow.utils.autologging_utils._AUTOLOGGING_GLOBALLY_DISABLED
+                or (
+                    mlflow.utils.autologging_utils._AUTOLOGGING_GLOBALLY_DISABLED
+                    and autologging_integration
+                )
             ):
                 # If the autologging integration associated with this patch is disabled,
                 # or if the current autologging integration is in exclusive mode and a user-created
@@ -575,9 +613,9 @@ def safe_patch(
                 if is_testing() and not preexisting_run_for_testing:
                     # If an MLflow run was created during the execution of patch code, verify that
                     # it is no longer active and that it contains expected autologging tags
-                    assert not mlflow.active_run(), (
-                        "Autologging integration %s leaked an active run" % autologging_integration
-                    )
+                    assert (
+                        not mlflow.active_run()
+                    ), f"Autologging integration {autologging_integration} leaked an active run"
                     if patch_function_run_for_testing:
                         _validate_autologging_run(
                             autologging_integration, patch_function_run_for_testing.info.run_id
@@ -633,13 +671,12 @@ def safe_patch(
             def bound_safe_patch_fn(*args, **kwargs):
                 return safe_patch_function(self, *args, **kwargs)
 
-            # Make bound method `instance.target_method` keep the same doc and signature
-            bound_safe_patch_fn = update_wrapper_extended(bound_safe_patch_fn, original_fn.fget)
+            # Make bound method `instance.target_method` keep the same doc and signature.
             # Here return the bound safe patch function because user call property decorated
             # method will like `instance.property_decorated_method(...)`, and internally it will
             # call the `bound_safe_patch_fn`, the argument list don't include the `self` argument,
             # so return bound function here.
-            return bound_safe_patch_fn
+            return update_wrapper_extended(bound_safe_patch_fn, original_fn.fget)
 
         # Make unbound method `class.target_method` keep the same doc and signature
         get_bound_safe_patch_fn = update_wrapper_extended(get_bound_safe_patch_fn, original_fn.fget)
@@ -652,14 +689,13 @@ def safe_patch(
 
 
 def revert_patches(autologging_integration):
-    """
-    Reverts all patches on the specified destination class for autologging disablement
-    purposes.
+    """Reverts all patches on the specified destination class for autologging disablement purposes.
+
+    Args:
+        autologging_integration: The name of the autologging integration associated with the
+            patch. Note: If called via fluent api (`autologging_integration="mlflow"`), then revert
+            all patches for all active autologging integrations.
 
-    :param autologging_integration: The name of the autologging integration associated with the
-                                    patch. Note: If called via fluent api
-                                    (`autologging_integration="mlflow"`), then revert all patches
-                                    for all active autologging integrations.
     """
     for patch in _AUTOLOGGING_PATCHES.get(autologging_integration, []):
         gorilla.revert(patch)
@@ -667,6 +703,15 @@ def revert_patches(autologging_integration):
     _AUTOLOGGING_PATCHES.pop(autologging_integration, None)
 
 
+def is_langchain_callbacks_manager(callbacks):
+    try:
+        from langchain_core.callbacks.base import BaseCallbackManager
+    except ImportError:
+        return False
+
+    return isinstance(callbacks, BaseCallbackManager)
+
+
 # Represents an active autologging session using two fields:
 # - integration: the name of the autologging integration corresponding to the session
 # - id: a unique session identifier (e.g., a UUID)
@@ -707,13 +752,13 @@ class _AutologgingSessionManager:
 
 
 def update_wrapper_extended(wrapper, wrapped):
-    """
-    Update a `wrapper` function to look like the `wrapped` function. This is an extension of
+    """Update a `wrapper` function to look like the `wrapped` function. This is an extension of
     `functools.update_wrapper` that applies the docstring *and* signature of `wrapped` to
     `wrapper`, producing a new function.
 
-    :return: A new function with the same implementation as `wrapper` and the same docstring
-             & signature as `wrapped`.
+    Returns:
+        A new function with the same implementation as `wrapper` and the same docstring
+        & signature as `wrapped`.
     """
     updated_wrapper = functools.update_wrapper(wrapper, wrapped)
     # Assign the signature of the `wrapped` function to the updated wrapper function.
@@ -727,14 +772,15 @@ def update_wrapper_extended(wrapper, wrapped):
 
 
 def _wrap_patch(destination, name, patch_obj, settings=None):
-    """
-    Apply a patch.
+    """Apply a patch.
+
+    Args:
+        destination: Patch destination.
+        name: Name of the attribute at the destination.
+        patch_obj: Patch object, it should be a function or a property decorated function
+            to be assigned to the patch point {destination}.{name}.
+        settings: Settings for gorilla.Patch.
 
-    :param destination: Patch destination
-    :param name: Name of the attribute at the destination
-    :param patch_obj: Patch object, it should be a function or a property decorated function
-                      to be assigned to the patch point {destination}.{name}
-    :param settings: Settings for gorilla.Patch
     """
     if settings is None:
         settings = gorilla.Settings(allow_hit=True, store_hit=True)
@@ -749,9 +795,10 @@ def _store_patch(autologging_integration, patch):
     Stores a patch for a specified autologging_integration class. Later to be used for being able
     to revert the patch when disabling autologging.
 
-    :param autologging_integration: The name of the autologging integration associated with the
-                                    patch.
-    :param patch: The patch to be stored.
+    Args:
+        autologging_integration: The name of the autologging integration associated with the
+            patch.
+        patch: The patch to be stored.
     """
     if autologging_integration in _AUTOLOGGING_PATCHES:
         _AUTOLOGGING_PATCHES[autologging_integration].add(patch)
@@ -776,7 +823,7 @@ def _validate_autologging_run(autologging_integration, run_id):
     )
     assert RunStatus.is_terminated(
         RunStatus.from_string(run.info.status)
-    ), "Autologging run with id {} has a non-terminal status '{}'".format(run_id, run.info.status)
+    ), f"Autologging run with id {run_id} has a non-terminal status '{run.info.status}'"
 
 
 class ValidationExemptArgument(typing.NamedTuple):
@@ -794,8 +841,8 @@ class ValidationExemptArgument(typing.NamedTuple):
     autologging_integration: str
     function_name: str
     type_function: typing.Callable
-    positional_argument_index: int = None
-    keyword_argument_name: str = None
+    positional_argument_index: Optional[int] = None
+    keyword_argument_name: Optional[str] = None
 
     def matches(
         self,
@@ -809,15 +856,18 @@ class ValidationExemptArgument(typing.NamedTuple):
         This method checks if the properties provided through the function arguments matches the
         properties defined in the NamedTuple.
 
-        :param autologging_integration: The name of an autologging integration.
-        :param function_name: The name of the function that is being matched.
-        :param value: The value of the argument.
-        :param argument_index: The index of the argument, if it is passed as a positional
-                                          argument. Otherwise it is None.
-        :param argument_name: The name of the argument, if it is passed as a keyword
-                                      argument. Otherwise it is None.
-        :return: Returns True if the given function properties matches the exempt argument's
-                 properties. Returns False otherwise.
+        Args:
+            autologging_integration: The name of an autologging integration.
+            function_name: The name of the function that is being matched.
+            value: The value of the argument.
+            argument_index: The index of the argument, if it is passed as a positional
+                argument. Otherwise it is None.
+            argument_name: The name of the argument, if it is passed as a keyword
+                argument. Otherwise it is None.
+
+        Returns:
+            Returns True if the given function properties matches the exempt argument's
+            properties. Returns False otherwise.
         """
         return (
             self.autologging_integration == autologging_integration
@@ -840,7 +890,8 @@ _VALIDATION_EXEMPT_ARGUMENTS = [
     #    custom generator class.
     # 2. The instance of `x` will be different, since we reconstructed the generator after consuming
     #    the first element.
-    ValidationExemptArgument("tensorflow", "fit", is_iterator, 1, "x")
+    ValidationExemptArgument("tensorflow", "fit", is_iterator, 1, "x"),
+    ValidationExemptArgument("keras", "fit", is_iterator, 1, "x"),
 ]
 
 
@@ -851,18 +902,20 @@ def _is_arg_exempt_from_validation(
     argument_index=None,
     argument_name=None,
 ):
-    """
-    This function is responsible for determining whether or not an argument is exempt from autolog
-    safety validations. This includes both type checking and immutable checking.
+    """This function is responsible for determining whether or not an argument is exempt from
+    autolog safety validations. This includes both type checking and immutable checking.
 
-    :param autologging_integration: The name of the autologging integration.
-    :param function_name: The name of the function that is being validated.
-    :param argument: The actual argument.
-    :param argument_index: The index of the argument, if it is passed as a positional
-                                      argument. Otherwise it is None.
-    :param argument_name: The name of the argument, if it is passed as a keyword argument.
-                                  Otherwise it is None.
-    :return: True or False
+    Args:
+        autologging_integration: The name of the autologging integration.
+        function_name: The name of the function that is being validated.
+        argument: The actual argument.
+        argument_index: The index of the argument, if it is passed as a positional
+            argument. Otherwise it is None.
+        argument_name: The name of the argument, if it is passed as a keyword argument.
+            Otherwise it is None.
+
+    Returns:
+        True or False
     """
     return any(
         exemption.matches(
@@ -910,6 +963,8 @@ def _validate_args(
         if type(inp) == list:
             for item in inp:
                 _validate_new_input(item)
+        elif isinstance(inp, dict) and "callbacks" in inp:
+            _validate_new_input(inp["callbacks"])
         elif callable(inp):
             assert getattr(inp, _ATTRIBUTE_EXCEPTION_SAFE, False), (
                 f"New function argument '{inp}' passed to original function is not exception-safe."
@@ -958,8 +1013,9 @@ def _validate_args(
             - for all other input types, `autologging_call_input` and `user_call_input`
               must be equivalent by reference equality or by object equality
 
-        :param autologging_call_input: call input from autologging
-        :param user_call_input: call input from user
+        Args:
+            autologging_call_input: call input from autologging.
+            user_call_input: call input from user.
         """
 
         if user_call_input is None and autologging_call_input is not None:
@@ -985,6 +1041,15 @@ def _validate_args(
             _assert_autologging_input_kwargs_are_superset(autologging_call_input, user_call_input)
             for key in autologging_call_input.keys():
                 _validate(autologging_call_input[key], user_call_input.get(key, None))
+        # NB: For LangChain autologging, we replace the callback manager that is passed by
+        # the user with the one we copied and injected our callbacks into. We cannot do this
+        # in-place to avoid side-effects, so create a new callback manager instance. It will
+        # fail at the strict equality check below, so we instead check that their handlers
+        # are effectively the compatible.
+        elif is_langchain_callbacks_manager(
+            autologging_call_input
+        ) and is_langchain_callbacks_manager(user_call_input):
+            _validate(autologging_call_input.handlers, user_call_input.handlers)
         else:
             assert (
                 autologging_call_input is user_call_input
@@ -1022,16 +1087,3 @@ def _validate_args(
                 autologging_call_kwargs[key],
                 user_call_kwargs.get(key, None),
             )
-
-
-__all__ = [
-    "safe_patch",
-    "is_testing",
-    "exception_safe_function_for_class",
-    "picklable_exception_safe_function",
-    "ExceptionSafeClass",
-    "ExceptionSafeAbstractClass",
-    "PatchFunction",
-    "with_managed_run",
-    "update_wrapper_extended",
-]
