@@ -1,30 +1,32 @@
-import os
 import logging
-import shutil
-import uuid
+import os
 import re
+import shutil
+import sys
+import tempfile
+import uuid
 from pathlib import Path
+from typing import Literal, Optional
+
 from packaging.version import Version
 
 import mlflow
+from mlflow.environment_variables import _MLFLOW_TESTING, MLFLOW_ENV_ROOT
 from mlflow.exceptions import MlflowException
-from mlflow.models.model import Model, MLMODEL_FILE_NAME
-from mlflow.utils.file_utils import TempDir
-from mlflow.utils.process import _exec_cmd, _join_commands, _IS_UNIX
-from mlflow.utils.requirements_utils import _parse_requirements
+from mlflow.models.model import MLMODEL_FILE_NAME, Model
+from mlflow.utils import env_manager as em
+from mlflow.utils.conda import _PIP_CACHE_DIR
 from mlflow.utils.environment import (
-    _PythonEnv,
-    _PYTHON_ENV_FILE_NAME,
     _CONDA_ENV_FILE_NAME,
+    _PYTHON_ENV_FILE_NAME,
     _REQUIREMENTS_FILE_NAME,
     _get_mlflow_env_name,
-    _get_pip_install_mlflow,
+    _PythonEnv,
 )
-from mlflow.utils.conda import _PIP_CACHE_DIR
-from mlflow.utils.databricks_utils import is_in_databricks_runtime
-
-_MLFLOW_ENV_ROOT_ENV_VAR = "MLFLOW_ENV_ROOT"
-
+from mlflow.utils.file_utils import remove_on_error
+from mlflow.utils.os import is_windows
+from mlflow.utils.process import _exec_cmd, _join_commands
+from mlflow.utils.requirements_utils import _parse_requirements
 
 _logger = logging.getLogger(__name__)
 
@@ -33,7 +35,7 @@ def _get_mlflow_virtualenv_root():
     """
     Returns the root directory to store virtualenv environments created by MLflow.
     """
-    return os.getenv(_MLFLOW_ENV_ROOT_ENV_VAR, str(Path.home().joinpath(".mlflow", "envs")))
+    return MLFLOW_ENV_ROOT.get()
 
 
 _DATABRICKS_PYENV_BIN_PATH = "/databricks/.pyenv/bin/pyenv"
@@ -43,10 +45,7 @@ def _is_pyenv_available():
     """
     Returns True if pyenv is available, otherwise False.
     """
-    if is_in_databricks_runtime():
-        return os.path.exists(_DATABRICKS_PYENV_BIN_PATH)
-    else:
-        return shutil.which("pyenv") is not None
+    return _get_pyenv_bin_path() is not None
 
 
 def _validate_pyenv_is_available():
@@ -56,7 +55,7 @@ def _validate_pyenv_is_available():
     """
     url = (
         "https://github.com/pyenv/pyenv#installation"
-        if _IS_UNIX
+        if not is_windows()
         else "https://github.com/pyenv-win/pyenv-win#installation"
     )
     if not _is_pyenv_available():
@@ -88,7 +87,9 @@ _SEMANTIC_VERSION_REGEX = re.compile(r"^([0-9]+)\.([0-9]+)\.([0-9]+)$")
 
 
 def _get_pyenv_bin_path():
-    return _DATABRICKS_PYENV_BIN_PATH if is_in_databricks_runtime() else "pyenv"
+    if os.path.exists(_DATABRICKS_PYENV_BIN_PATH):
+        return _DATABRICKS_PYENV_BIN_PATH
+    return shutil.which("pyenv")
 
 
 def _find_latest_installable_python_version(version_prefix):
@@ -98,7 +99,9 @@ def _find_latest_installable_python_version(version_prefix):
     where 'x' represents the latest micro version in 3.8.
     """
     lines = _exec_cmd(
-        [_get_pyenv_bin_path(), "install", "--list"], capture_output=True
+        [_get_pyenv_bin_path(), "install", "--list"],
+        capture_output=True,
+        shell=is_windows(),
     ).stdout.splitlines()
     semantic_versions = filter(_SEMANTIC_VERSION_REGEX.match, map(str.strip, lines))
     matched = [v for v in semantic_versions if v.startswith(version_prefix)]
@@ -108,15 +111,17 @@ def _find_latest_installable_python_version(version_prefix):
 
 
 def _install_python(version, pyenv_root=None, capture_output=False):
-    """
-    Installs a specified version of python with pyenv and returns a path to the installed python
+    """Installs a specified version of python with pyenv and returns a path to the installed python
     binary.
 
-    :param version: Python version to install.
-    :param pyenv_root: The value of the "PYENV_ROOT" environment variable used when running
-                       `pyenv install` which installs python in `{PYENV_ROOT}/versions/{version}`.
-    :param capture_output: Set the `capture_output` argument when calling `_exec_cmd`
-    :return: Path to the installed python binary.
+    Args:
+        version: Python version to install.
+        pyenv_root: The value of the "PYENV_ROOT" environment variable used when running
+            `pyenv install` which installs python in `{PYENV_ROOT}/versions/{version}`.
+        capture_output: Set the `capture_output` argument when calling `_exec_cmd`.
+
+    Returns:
+        Path to the installed python binary.
     """
     version = (
         version
@@ -126,18 +131,18 @@ def _install_python(version, pyenv_root=None, capture_output=False):
     _logger.info("Installing python %s if it does not exist", version)
     # pyenv-win doesn't support `--skip-existing` but its behavior is enabled by default
     # https://github.com/pyenv-win/pyenv-win/pull/314
-    pyenv_install_options = ("--skip-existing",) if _IS_UNIX else ()
+    pyenv_install_options = ("--skip-existing",) if not is_windows() else ()
     extra_env = {"PYENV_ROOT": pyenv_root} if pyenv_root else None
     pyenv_bin_path = _get_pyenv_bin_path()
     _exec_cmd(
         [pyenv_bin_path, "install", *pyenv_install_options, version],
         capture_output=capture_output,
         # Windows fails to find pyenv and throws `FileNotFoundError` without `shell=True`
-        shell=not _IS_UNIX,
+        shell=is_windows(),
         extra_env=extra_env,
     )
 
-    if _IS_UNIX:
+    if not is_windows():
         if pyenv_root is None:
             pyenv_root = _exec_cmd([pyenv_bin_path, "root"], capture_output=True).stdout.strip()
         path_to_bin = ("bin", "python")
@@ -176,15 +181,18 @@ def _get_python_env_file(model_config):
 
 
 def _get_python_env(local_model_path):
-    """
-    Constructs `_PythonEnv` from the model artifacts stored in `local_model_path`. If
+    """Constructs `_PythonEnv` from the model artifacts stored in `local_model_path`. If
     `python_env.yaml` is available, use it, otherwise extract model dependencies from `conda.yaml`.
     If `conda.yaml` contains conda dependencies except `python`, `pip`, `setuptools`, and, `wheel`,
     an `MlflowException` is thrown because conda dependencies cannot be installed in a virtualenv
     environment.
 
-    :param local_model_path: Local directory containing the model artifacts.
-    :return: `_PythonEnv` instance.
+    Args:
+        local_model_path: Local directory containing the model artifacts.
+
+    Returns:
+        `_PythonEnv` instance.
+
     """
     model_config = Model.load(local_model_path / MLMODEL_FILE_NAME)
     python_env_file = local_model_path / _get_python_env_file(model_config)
@@ -220,55 +228,113 @@ def _get_virtualenv_name(python_env, work_dir_path, env_id=None):
         base_dir=work_dir_path,
     )
     return _get_mlflow_env_name(
-        str(python_env) + "".join(x.req_str for x in requirements) + (env_id or "")
+        str(python_env) + "".join(map(str, sorted(requirements))) + (env_id or "")
     )
 
 
-def _create_virtualenv(
-    local_model_path, python_bin_path, env_dir, python_env, extra_env=None, capture_output=False
-):
+def _get_virtualenv_activate_cmd(env_dir: Path) -> str:
     # Created a command to activate the environment
-    paths = ("bin", "activate") if _IS_UNIX else ("Scripts", "activate.bat")
+    paths = ("bin", "activate") if not is_windows() else ("Scripts", "activate.bat")
     activate_cmd = env_dir.joinpath(*paths)
-    activate_cmd = f"source {activate_cmd}" if _IS_UNIX else activate_cmd
+    return f"source {activate_cmd}" if not is_windows() else str(activate_cmd)
 
+
+def _create_virtualenv(
+    local_model_path: Path,
+    python_env: _PythonEnv,
+    env_dir: Path,
+    pyenv_root_dir: Optional[str] = None,
+    env_manager: Literal["virtualenv", "uv"] = em.UV,
+    extra_env: Optional[dict[str, str]] = None,
+    capture_output: bool = False,
+    pip_requirements_override: Optional[list[str]] = None,
+):
+    if env_manager not in {em.VIRTUALENV, em.UV}:
+        raise MlflowException.invalid_parameter_value(
+            f"Invalid value for `env_manager`: {env_manager}. "
+            f"Must be one of `{em.VIRTUALENV}, {em.UV}`"
+        )
+
+    activate_cmd = _get_virtualenv_activate_cmd(env_dir)
     if env_dir.exists():
-        _logger.info("Environment %s already exists", env_dir)
+        _logger.info(f"Environment {env_dir} already exists")
         return activate_cmd
 
-    _logger.info("Creating a new environment in %s with %s", env_dir, python_bin_path)
-    _exec_cmd(["virtualenv", "--python", python_bin_path, env_dir], capture_output=capture_output)
+    if env_manager == em.VIRTUALENV:
+        python_bin_path = _install_python(
+            python_env.python, pyenv_root=pyenv_root_dir, capture_output=capture_output
+        )
+        _logger.info(f"Creating a new environment in {env_dir} with {python_bin_path}")
+        env_creation_cmd = [
+            sys.executable,
+            "-m",
+            "virtualenv",
+            "--python",
+            python_bin_path,
+            env_dir,
+        ]
+        install_deps_cmd_prefix = "python -m pip install"
+    elif env_manager == em.UV:
+        _logger.info(
+            f"Creating a new environment in {env_dir} with python "
+            f"version {python_env.python} using uv"
+        )
+        env_creation_cmd = ["uv", "venv", env_dir, f"--python={python_env.python}"]
+        install_deps_cmd_prefix = "uv pip install --prerelease=allow"
+        if _MLFLOW_TESTING.get():
+            os.environ["RUST_LOG"] = "uv=debug"
+    with remove_on_error(
+        env_dir,
+        onerror=lambda e: _logger.warning(
+            "Encountered an unexpected error: %s while creating a virtualenv environment in %s, "
+            "removing the environment directory...",
+            repr(e),
+            env_dir,
+        ),
+    ):
+        _exec_cmd(
+            env_creation_cmd,
+            capture_output=capture_output,
+        )
 
-    _logger.info("Installing dependencies")
-    for deps in filter(None, [python_env.build_dependencies, python_env.dependencies]):
-        with TempDir() as t:
-            # Create a temporary requirements file in the model directory to resolve the references
-            # in it correctly. To do this, we must first symlink or copy the model directory's
-            # contents to a temporary location for compatibility with deployment tools that store
-            # models in a read-only mount
-            tmp_model_dir = t.path("model")
-            os.makedirs(tmp_model_dir)
-            try:
-                for model_item in os.listdir(local_model_path):
-                    os.symlink(
-                        src=os.path.join(local_model_path, model_item),
-                        dst=os.path.join(tmp_model_dir, model_item),
+        _logger.info("Installing dependencies")
+        for deps in filter(None, [python_env.build_dependencies, python_env.dependencies]):
+            with tempfile.TemporaryDirectory() as tmpdir:
+                # Create a temporary requirements file in the model directory to resolve the
+                # references in it correctly. To do this, we must first symlink or copy the model
+                # directory's contents to a temporary location for compatibility with deployment
+                # tools that store models in a read-only mount
+                try:
+                    for model_item in os.listdir(local_model_path):
+                        os.symlink(
+                            src=os.path.join(local_model_path, model_item),
+                            dst=os.path.join(tmpdir, model_item),
+                        )
+                except Exception as e:
+                    _logger.warning(
+                        "Failed to symlink model directory during dependency installation"
+                        " Copying instead. Exception: %s",
+                        e,
                     )
-            except Exception as e:
-                _logger.warning(
-                    "Failed to symlink model directory during dependency installation"
-                    " Copying instead. Exception: %s",
-                    e,
-                )
-                shutil.rmtree(tmp_model_dir)
-                _copy_model_to_writeable_destination(local_model_path, tmp_model_dir)
+                    _copy_model_to_writeable_destination(local_model_path, tmpdir)
 
-            tmp_req_file = f"requirements.{uuid.uuid4().hex}.txt"
-            Path(tmp_model_dir).joinpath(tmp_req_file).write_text("\n".join(deps))
-            cmd = _join_commands(activate_cmd, f"python -m pip install --quiet -r {tmp_req_file}")
-            _exec_cmd(cmd, capture_output=capture_output, cwd=tmp_model_dir, extra_env=extra_env)
+                tmp_req_file = f"requirements.{uuid.uuid4().hex}.txt"
+                Path(tmpdir).joinpath(tmp_req_file).write_text("\n".join(deps))
+                cmd = _join_commands(activate_cmd, f"{install_deps_cmd_prefix} -r {tmp_req_file}")
+                _exec_cmd(cmd, capture_output=capture_output, cwd=tmpdir, extra_env=extra_env)
 
-    return activate_cmd
+        if pip_requirements_override:
+            _logger.info(
+                "Installing additional dependencies specified by "
+                f"pip_requirements_override: {pip_requirements_override}"
+            )
+            cmd = _join_commands(
+                activate_cmd,
+                f"{install_deps_cmd_prefix} --quiet {' '.join(pip_requirements_override)}",
+            )
+            _exec_cmd(cmd, capture_output=capture_output, extra_env=extra_env)
+
+        return activate_cmd
 
 
 def _copy_model_to_writeable_destination(model_src, dst):
@@ -281,10 +347,7 @@ def _copy_model_to_writeable_destination(model_src, dst):
         # Copy individual files and subdirectories, rather than using `shutil.copytree()`
         # because `shutil.copytree()` will apply the permissions from the source directory,
         # which may be read-only
-        if os.path.isdir(model_item):
-            copy_fn = shutil.copytree
-        else:
-            copy_fn = shutil.copy2
+        copy_fn = shutil.copytree if os.path.isdir(model_item) else shutil.copy2
 
         copy_fn(
             src=os.path.join(model_src, model_item),
@@ -308,101 +371,73 @@ _VIRTUALENV_ENVS_DIR = "virtualenv_envs"
 _PYENV_ROOT_DIR = "pyenv_root"
 
 
-def _get_or_create_virtualenv(
-    local_model_path, env_id=None, env_root_dir=None, capture_output=False
+def _get_or_create_virtualenv(  # noqa: D417
+    local_model_path,
+    env_id=None,
+    env_root_dir=None,
+    capture_output=False,
+    pip_requirements_override: Optional[list[str]] = None,
+    env_manager: Literal["virtualenv", "uv"] = em.UV,
 ):
-    """
-    Restores an MLflow model's environment with pyenv and virtualenv and returns a command
+    """Restores an MLflow model's environment in a virtual environment and returns a command
     to activate it.
 
-    :param local_model_path: Local directory containing the model artifacts.
-    :param env_id: Optional string that is added to the contents of the yaml file before
-                   calculating the hash. It can be used to distinguish environments that have the
-                   same conda dependencies but are supposed to be different based on the context.
-                   For example, when serving the model we may install additional dependencies to the
-                   environment after the environment has been activated.
-    :return: Command to activate the created virtualenv environment
-             (e.g. "source /path/to/bin/activate").
+    Args:
+        local_model_path: Local directory containing the model artifacts.
+        env_id: Optional string that is added to the contents of the yaml file before
+            calculating the hash. It can be used to distinguish environments that have the
+            same conda dependencies but are supposed to be different based on the context.
+            For example, when serving the model we may install additional dependencies to the
+            environment after the environment has been activated.
+        pip_requirements_override: If specified, install the specified python dependencies to
+            the environment (upgrade if already installed).
+        env_manager: Specifies the environment manager to use to create the environment.
+            Defaults to "uv".
+
+            .. tip::
+                It is highly recommended to use "uv" as it has significant performance improvements
+                over "virtualenv".
+
+    Returns:
+        Command to activate the created virtual environment
+        (e.g. "source /path/to/bin/activate").
+
     """
-    _validate_pyenv_is_available()
-    _validate_virtualenv_is_available()
+    if env_manager == em.VIRTUALENV:
+        _validate_pyenv_is_available()
+        _validate_virtualenv_is_available()
 
-    # Read environment information
     local_model_path = Path(local_model_path)
     python_env = _get_python_env(local_model_path)
 
-    extra_env = _get_virtualenv_extra_env_vars(env_root_dir)
-    if env_root_dir is not None:
-        virtual_envs_root_path = Path(env_root_dir) / _VIRTUALENV_ENVS_DIR
-        pyenv_root_path = Path(env_root_dir) / _PYENV_ROOT_DIR
-        pyenv_root_path.mkdir(parents=True, exist_ok=True)
-        pyenv_root_dir = str(pyenv_root_path)
-    else:
+    pyenv_root_dir = None
+    if env_root_dir is None:
         virtual_envs_root_path = Path(_get_mlflow_virtualenv_root())
-        pyenv_root_dir = None
+    else:
+        virtual_envs_root_path = Path(env_root_dir) / _VIRTUALENV_ENVS_DIR
+        if env_manager == em.VIRTUALENV:
+            pyenv_root_path = Path(env_root_dir) / _PYENV_ROOT_DIR
+            pyenv_root_path.mkdir(parents=True, exist_ok=True)
+            pyenv_root_dir = str(pyenv_root_path)
 
-    virtual_envs_root_path.mkdir(parents=True, exist_ok=True)
-
-    # Create an environment
-    python_bin_path = _install_python(
-        python_env.python, pyenv_root=pyenv_root_dir, capture_output=capture_output
+    virtual_envs_root_path = (
+        Path(env_root_dir) / _VIRTUALENV_ENVS_DIR
+        if env_root_dir is not None
+        else Path(_get_mlflow_virtualenv_root())
     )
+    virtual_envs_root_path.mkdir(parents=True, exist_ok=True)
     env_name = _get_virtualenv_name(python_env, local_model_path, env_id)
     env_dir = virtual_envs_root_path / env_name
-    try:
-        return _create_virtualenv(
-            local_model_path,
-            python_bin_path,
-            env_dir,
-            python_env,
-            extra_env=extra_env,
-            capture_output=capture_output,
-        )
-    except:
-        _logger.warning("Encountered unexpected error while creating %s", env_dir)
-        if env_dir.exists():
-            _logger.warning("Attempting to remove %s", env_dir)
-            shutil.rmtree(env_dir, ignore_errors=True)
-            msg = "Failed to remove %s" if env_dir.exists() else "Successfully removed %s"
-            _logger.warning(msg, env_dir)
-        raise
+    extra_env = _get_virtualenv_extra_env_vars(env_root_dir)
 
-
-def _execute_in_virtualenv(
-    activate_cmd,
-    command,
-    install_mlflow,
-    command_env=None,
-    synchronous=True,
-    capture_output=False,
-    env_root_dir=None,
-    **kwargs,
-):
-    """
-    Runs a command in a specified virtualenv environment.
-
-    :param activate_cmd: Command to activate the virtualenv environment.
-    :param command: Command to run in the virtualenv environment.
-    :param install_mlflow: Flag to determine whether to install mlflow in the virtualenv
-                           environment.
-    :param command_env: Environment variables passed to a process running the command.
-    :param synchronous: Set the `synchronous` argument when calling `_exec_cmd`.
-    :param capture_output: Set the `capture_output` argument when calling `_exec_cmd`.
-    :param env_root_dir: See doc of PyFuncBackend constructor argument `env_root_dir`.
-    :param kwargs: Set the `kwargs` argument when calling `_exec_cmd`
-    """
-    if command_env is None:
-        command_env = os.environ.copy()
-
-    if env_root_dir is not None:
-        command_env = {**command_env, **_get_virtualenv_extra_env_vars(env_root_dir)}
-
-    pre_command = [activate_cmd]
-    if install_mlflow:
-        pre_command.append(_get_pip_install_mlflow())
-
-    cmd = _join_commands(*pre_command, command)
-    _logger.info("Running command: %s", " ".join(cmd))
-    return _exec_cmd(
-        cmd, capture_output=capture_output, env=command_env, synchronous=synchronous, **kwargs
+    # Create an environment
+    return _create_virtualenv(
+        local_model_path=local_model_path,
+        python_env=python_env,
+        env_dir=env_dir,
+        pyenv_root_dir=pyenv_root_dir,
+        env_manager=env_manager,
+        extra_env=extra_env,
+        capture_output=capture_output,
+        pip_requirements_override=pip_requirements_override,
     )
